/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>

#include <GL/gl.h>

#include "config.h"
#include "camera.h"

camera::camera()
{
	width = WIDTH;
	height = HEIGHT;
	nearv.set_z(NNEAR);
	farv.set_z(FFAR);
	set_fov(FOV);
}

void camera::set_size(float w, float h)
{
	width = w;
	height = h;
	set_fov(fov);
}

void camera::set_fov(float f)
{
	float fx, fy;

	fov = f;

	if (height > width)
	{
		fx = f * width / height;
		fy = f;
	}
	else
	{
		fx = f;
		fy = f * height / width;
	}

	nearv.set_x(nearv.z() * fx);
	farv.set_x(farv.z() * fx);

	nearv.set_y(nearv.z() * fy);
	farv.set_y(farv.z() * fy);
}

#include <stdio.h>

void camera::perspective()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-nearv.x(), nearv.x(), -nearv.y(), nearv.y(),
		  nearv.z(), farv.z());
	glMatrixMode(GL_MODELVIEW);
}

void camera::ortho()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, 0, height, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void camera::min_max(vector *v)
{
	v->rotate(&ctow);
	box.min_max(v);
}

void camera::load()
{
	vector v;

	wtoc.set();
	wtoc.rotate_x(tilt + 0.5);
	wtoc.rotate_z(pan);
	v.neg(&pos); wtoc.translate(&v);
	ctow.invert(&wtoc);
	wtoc.glload();

	box.min.set(&pos);
	box.max.set(&pos);

	v.set(farv.x(), farv.y(), -farv.z()); min_max(&v);
	v.set(-farv.x(), farv.y(), -farv.z()); min_max(&v);
	v.set(farv.x(), -farv.y(), -farv.z()); min_max(&v);
	v.set(-farv.x(), -farv.y(), -farv.z()); min_max(&v);
}

void camera::screen_vector(vector *v, float x, float y)
{
	vector p;

	v->set(nearv.x() * ((x * 2.0 / width) - 1.0),
	       -nearv.y() * ((y * 2.0 / height) - 1.0),
	       -nearv.z());
	v->rotate(&ctow);
	position(&p);
	v->sub(&p);
}
