/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CAMERA_H__
#define __CAMERA_H__

#include "matrix.h"
#include "bbox.h"

class camera
{
 private:
	float width, height, fov;
	vector nearv, farv;
	matrix wtoc, ctow;

 public:
	vector pos;
	bbox box;
	float pan, tilt;

	camera();
	void perspective();
	void ortho();
	void load();

	void set_size(float, float);
	void set_fov(float);

	float get_width() { return width; }
	float get_height() { return height; }

	void screen_vector(vector *, float, float);

	inline void sideways(vector *v)
		{ v->get_x(&ctow); }
	inline void upward(vector *v)
		{ v->get_y(&ctow); }
	inline void backward(vector *v)
		{ v->get_z(&ctow); }
	inline void position(vector *v)
		{ v->get_w(&ctow); }

 private:
	void min_max(vector *);
};

#endif
