/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>

#include "gui.h"
#include "config.h"

static gui *menu_gui;

static void menu_func(int value)
{
	switch (value)
	{
	case 1:
		menu_gui->screen_shot();
		break;
	case 2:
		menu_gui->save_map();
		break;
	case 3:
		exit(0);
		break;

	case 4:
		menu_gui->map.unselect_square();
		menu_gui->cancel();
		break;
	case 5:
		menu_gui->new_triangle();
		break;
	case 6:
		menu_gui->select_triangle();
		break;
	case 7:
		menu_gui->rotate_block();
		break;
	case 8:
		menu_gui->copy();
		break;
	case 9:
		menu_gui->cut();
		break;
	case 10:
		menu_gui->paste();
		break;
	case 11:
		menu_gui->map.unselect_triangle();
		menu_gui->cancel();
		break;
	case 12:
		menu_gui->delete_triangle();
		break;
	case 13:
		menu_gui->select_texture();
		break;
	case 14:
		menu_gui->move_xy = 1;
		break;
	case 15:
		menu_gui->move_xy = 0;
		break;
	}
	glutPostRedisplay();
}

gui::gui()
{
	cam = 0;
	menu = 0;
	file_menu = 0;
	block_menu = 0;
	tri_menu = 0;
	draw_fn = 0;
	keyboard_fn = 0;
	mouse_up_fn = 0;
	mouse_down_fn = 0;
	motion_fn = 0;
	filename = 0;

	new_tri = 0;
	move_xy = 0;

	str[0] = '\0';
}

gui::~gui()
{
	if (menu)
		glutDestroyMenu(menu);
	if (file_menu)
		glutDestroyMenu(file_menu);
	if (block_menu)
		glutDestroyMenu(block_menu);
}

void gui::init(camera *c)
{
	cam = c;

	file_menu = glutCreateMenu(menu_func);
	glutAddMenuEntry("Screen Shot", 1);
	glutAddMenuEntry("Save", 2);
	glutAddMenuEntry("Quit", 3);

	block_menu = glutCreateMenu(menu_func);
	glutAddMenuEntry("Unselect", 4);
	glutAddMenuEntry("New Triangle", 5);
	glutAddMenuEntry("Select Triangle", 6);
	glutAddMenuEntry("Rotate", 7);
	glutAddMenuEntry("Copy", 8);
	glutAddMenuEntry("Cut", 9);
	glutAddMenuEntry("Paste", 10);

	tri_menu = glutCreateMenu(menu_func);
	glutAddMenuEntry("Unselect", 11);
	glutAddMenuEntry("Delete", 12);
	glutAddMenuEntry("Select Texture", 13);
	glutAddMenuEntry("Move XY", 14);
	glutAddMenuEntry("Move Z", 15);

	cancel();
}

void gui::draw_ui()
{
	char c, *s;

	if (draw_fn) (this->*draw_fn)();

	glDisable(GL_TEXTURE_2D);

	glRasterPos2i(2, int(cam->get_height()) - 10);

	for (s = str; (c = *s); s++)
	{
		glutBitmapCharacter(FONT, c);
	}
}

void gui::cancel()
{
	if (new_tri)
	{
		map.delete_triangle(new_tri);
		new_tri = 0;
	}

	str[0] = '\0';

	draw_fn = 0;
	keyboard_fn = &gui::edit_keys;

	set_menu(1);

	if (!map.selected_square())
	{
		select_block();
	}
	else if (!map.selected_triangle())
	{
		move_block();
	}
	else
	{
		move_vertex();
	}
}

void gui::set_menu(int i)
{
	if (menu) glutDestroyMenu(menu);

	menu = glutCreateMenu(menu_func);
	glutAddSubMenu("File", file_menu);

	if (i)
	{
		if (map.selected_square())
			glutAddSubMenu("Block", block_menu);

		if (map.selected_triangle())
			glutAddSubMenu("Triangle", tri_menu);
	}

	glutAttachMenu(GLUT_RIGHT_BUTTON);

	menu_gui = this;
}

void gui::edit_keys(int key, int, int)
{
	vector v;

	switch (key)
	{
	case '\b':
		delete_triangle();
		break;
	case '\t':
		map.select_next_triangle();
		// fall though;
	case '\e':
		cancel();
		break;

	case ' ':
		map.unselect_triangle();
		cancel();
		break;
	case 'C':
	case 'c':
		copy();
		break;
	case 'F':
	case 'f':
		glutFullScreen();
		break;
	case 'N':
	case 'n':
		new_triangle();
		break;
	case 'R':
	case 'r':
		rotate_block();
		break;
	case 'S':
	case 's':
		select_triangle();
		break;
	case 'T':
	case 't':
		select_texture();
		break;
	case 'U':
	case 'u':
		map.unselect_square();
		cancel();
		break;
	case 'V':
	case 'v':
		paste();
		break;
	case 'X':
	case 'x':
		move_xy = 1;
		break;
	case 'Z':
	case 'z':
		move_xy = 0;
		break;

	case -GLUT_KEY_LEFT:
		cam->sideways(&v);
		cam->pos.sub(&v);
		break;
	case -GLUT_KEY_UP:
		cam->backward(&v);
		cam->pos.sub(&v);
		break;
	case -GLUT_KEY_RIGHT:
		cam->sideways(&v);
		cam->pos.add(&v);
		break;
	case -GLUT_KEY_DOWN:
		cam->backward(&v);
		cam->pos.add(&v);
		break;
	case -GLUT_KEY_PAGE_UP:
		cam->upward(&v);
		cam->pos.add(&v);
		break;
	case -GLUT_KEY_PAGE_DOWN:
		cam->upward(&v);
		cam->pos.sub(&v);
		break;
	case -GLUT_KEY_HOME:
		cam->tilt -= 0.0078125;
		break;
	case -GLUT_KEY_END:
		cam->tilt += 0.0078125;
		break;
	case -GLUT_KEY_INSERT:
		cam->pan -= 0.0078125;
		break;
	case '\177':
		cam->pan += 0.0078125;
		break;
	}
	glutPostRedisplay();
}

void gui::select_block()
{
	mouse_down_fn = &gui::select_block_down;
	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::select_block_down(int x, int y)
{
	vector pos, dir;

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));
	map.select_square(&pos, &dir, &cam->box);

	mouse_down_fn = &gui::select_block_down;
	mouse_up_fn = &gui::select_block_up;
	motion_fn = &gui::select_block_down;

	glutPostRedisplay();
}

void gui::select_block_up(int x, int y)
{
	vector pos, dir;

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));
	map.select_square(&pos, &dir, &cam->box);

	cancel();

	glutPostRedisplay();
}

void gui::move_block()
{
	if (map.selected_square())
	{
		mouse_down_fn = &gui::move_block_down;
		mouse_up_fn = 0;
		motion_fn = 0;
	}
}

void gui::move_block_down(int x, int y)
{
	vector pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (vec.isect_z_line(sqr, &pos, &dir) > 0)
	{
		print_height(int(sqr->z()));

		mouse_up_fn = &gui::move_block_up;
		motion_fn = &gui::move_block_motion;
	}
}

void gui::move_block_up(int x, int y)
{
	move_block_motion(x, y);
	str[0] = '\0';
	glutPostRedisplay();

	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::move_block_motion(int x, int y)
{
	vector v, pos, dir;

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_xy_line(&vec, &pos, &dir) > 0)
	{
		square *sqr = map.selected_square();

		sqr->set_height(v.z());
		print_height(int(sqr->z()));
	}
}

void gui::print_height(int h)
{
	sprintf(str, "%d", h);
	glutPostRedisplay();
}

void gui::new_triangle()
{
	square *sqr = map.selected_square();

	if (sqr && sqr->triangles() < 255)
	{
		mouse_down_fn = &gui::new_triangle_down1;
		mouse_up_fn = 0;
		motion_fn = 0;
	}
}

void gui::new_triangle_down1(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri = map.new_triangle(&v);
		print_triangle1(new_tri);

		mouse_down_fn = &gui::new_triangle_down2;
		mouse_up_fn = &gui::new_triangle_up1;
		motion_fn = &gui::new_triangle_motion1;
	}
}

void gui::new_triangle_up1(int x, int y)
{
	new_triangle_motion1(x, y);

	mouse_down_fn = &gui::new_triangle_down2;
	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::new_triangle_motion1(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri->vtx[0].set(&v);
		new_tri->vtx[1].set(&v);
		new_tri->vtx[2].set(&v);
		print_triangle1(new_tri);
	}

}

void gui::print_triangle1(triangle *tri)
{
	sprintf(str, "(%d,%d)",
		int(tri->vtx[0].x()), int(tri->vtx[0].y()));
	glutPostRedisplay();
}

void gui::new_triangle_down2(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri->vtx[1].set(&v);
		print_triangle2(new_tri);

		mouse_down_fn = &gui::new_triangle_down3;
		mouse_up_fn = &gui::new_triangle_up2;
		motion_fn = &gui::new_triangle_motion2;
	}
}

void gui::new_triangle_up2(int x, int y)
{
	new_triangle_motion2(x, y);

	mouse_down_fn = &gui::new_triangle_down3;
	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::new_triangle_motion2(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri->vtx[1].set(&v);
		print_triangle2(new_tri);
	}
}

void gui::print_triangle2(triangle *tri)
{
	sprintf(str, "(%d,%d)-(%d,%d)",
		int(tri->vtx[0].x()), int(tri->vtx[0].y()),
		int(tri->vtx[1].x()), int(tri->vtx[1].y()));
	glutPostRedisplay();
}

void gui::new_triangle_down3(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri->vtx[2].set(&v);
		print_triangle3(new_tri);

		mouse_down_fn = 0;
		mouse_up_fn = &gui::new_triangle_up3;
		motion_fn = &gui::new_triangle_motion3;
	}
}

void gui::new_triangle_up3(int x, int y)
{
	new_triangle_motion3(x, y);

	new_tri = 0;
	cancel();
}

void gui::new_triangle_motion3(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if (v.isect_z_line(sqr, &pos, &dir) > 0)
	{
		sqr->fix_vec(&v);
		new_tri->vtx[2].set(&v);
		print_triangle3(new_tri);
	}
}

void gui::print_triangle3(triangle *tri)
{
	sprintf(str, "(%d,%d)-(%d,%d)-(%d,%d)",
		int(tri->vtx[0].x()), int(tri->vtx[0].y()),
		int(tri->vtx[1].x()), int(tri->vtx[1].y()),
		int(tri->vtx[2].x()), int(tri->vtx[2].y()));
	glutPostRedisplay();
}

void gui::select_triangle()
{
	if (map.selected_square())
	{
		mouse_down_fn = &gui::select_triangle_down;
		mouse_up_fn = 0;
		motion_fn = 0;
	}
}

void gui::select_triangle_down(int x, int y)
{
	vector pos, dir;

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));
	map.select_triangle(&pos, &dir);

	mouse_down_fn = &gui::select_triangle_down;
	mouse_up_fn = &gui::select_triangle_up;
	motion_fn = &gui::select_triangle_down;

	glutPostRedisplay();
}

void gui::select_triangle_up(int x, int y)
{
	vector pos, dir;

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));
	map.select_triangle(&pos, &dir);

	cancel();
	glutPostRedisplay();
}

void gui::delete_triangle()
{
	triangle *tri = map.selected_triangle();

	if (tri)
	{
		map.delete_triangle(tri);
		cancel();
	}
}

void gui::move_vertex()
{
	if (map.selected_triangle())
	{
		mouse_down_fn = &gui::move_vertex_down;
		mouse_up_fn = 0;
		motion_fn = 0;
	}
}

void gui::move_vertex_down(int x, int y)
{
	vector pos, dir;
	square *sqr = map.selected_square();
	triangle *tri = map.selected_triangle();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if ((vtx = tri->select_vertex(sqr, &pos, &dir)) >= 0)
	{
		vec.add(sqr, &tri->vtx[vtx]);
		print_vertex(&tri->vtx[vtx]);

		mouse_up_fn = &gui::move_vertex_up;
		motion_fn = &gui::move_vertex_motion;
	}
}

void gui::move_vertex_up(int x, int y)
{
	move_vertex_motion(x, y);
	str[0] = '\0';
	glutPostRedisplay();

	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::move_vertex_motion(int x, int y)
{
	vector v, pos, dir;
	square *sqr = map.selected_square();

	cam->position(&pos);
	cam->screen_vector(&dir, float(x), float(y));

	if ( (move_xy ? v.isect_z_line(&vec, &pos, &dir) :
	      v.isect_xy_line(&vec, &pos, &dir)) > 0 )
	{
		triangle *tri = map.selected_triangle();

		sqr->fix_vec(&v);
		tri->vtx[vtx].set(&v);
		print_vertex(&tri->vtx[vtx]);
	}
}

void gui::print_vertex(vector *v)
{
	sprintf(str, "%d,%d,%d",
		int(v->x()), int(v->y()), int(v->z()));
	glutPostRedisplay();
}

void gui::rotate_block()
{
	square *sqr = map.selected_square();

	if (!sqr) return;

	for (triangle *tri = sqr->tris; tri; tri = tri->next)
	{
		for (unsigned i = 0; i < 3; i++)
		{
			vector *v = &tri->vtx[i];
			float x = v->x();
			float y = v->y();

			v->set_x(y);
			v->set_y(16.0 - x);
		}
	}
}

void gui::copy()
{
	square *sqr = map.selected_square();
	
	if (sqr) buf.copy(sqr);
}

void gui::cut()
{
	square *sqr = map.selected_square();

	if (sqr)
	{
		buf.copy(sqr);
		sqr->clear();
		map.unselect_triangle();
		cancel();
	}
}

void gui::paste()
{
	square *sqr = map.selected_square();

	if (sqr)
	{
		sqr->copy(&buf);
		map.unselect_triangle();
		cancel();
	}
}

void gui::select_texture()
{
	if (map.selected_triangle())
	{
		set_menu(0);

		draw_fn = &gui::select_texture_draw;
		keyboard_fn = &gui::select_texture_keys;
		mouse_down_fn = &gui::select_texture_down;
		mouse_up_fn = 0;
		motion_fn = 0;

		tn = 1;
		select_texture_calcsize();
	}
}

void gui::select_texture_calcsize()
{
	triangle *tri = map.selected_triangle();
	float w = cam->get_width();
	float h = cam->get_height();
	float f;

	for (unsigned i = 0; i < 3; i++)
	{
		if (tri->vtx[i].u > 1.0 || tri->vtx[i].v > 1.0)
		{
			tn = 3;
			break;
		}
	}

	f = h;
	if (w < h) f = w;
	f = (f - 20.0) / 192.0;
	ts = int(f) * 192;

	f = float(ts);
	tx = int((w - f) / 2.0);
	ty = int((h - f) / 2.0);
}

void gui::select_texture_draw()
{
	int x1, x2, y1, y2;
	triangle *tri = map.selected_triangle();
	char c;
	const char *s;

	select_texture_calcsize();

	x1 = tx - 1; x2 = tx + ts + 1;
	y1 = ty + ts + 1; y2 = ty - 17;

	glDisable(GL_TEXTURE_2D);
	glColor3ub(255, 255, 255);

	glBegin(GL_QUADS);
	glVertex2i(x1, y1);
	glVertex2i(x1, y2);
	glVertex2i(x2, y2);
	glVertex2i(x2, y1);
	glEnd();

	x1 += 1; x2 -= 1;
	y2 += 1;
	y1 = y2 + 15;

	glColor3ub(0, 0, 0);

	glBegin(GL_QUADS);
	glVertex2i(x1, y1);
	glVertex2i(x1, y2);
	glVertex2i(x2, y2);
	glVertex2i(x2, y1);
	glEnd();

	glColor3ub(255, 255, 255);

	if ( (s = tri->tex.name()) )
	{
		glRasterPos2i(x1 + 2, y2 + 3);

		while ( (c = *(s++)) )
		{
			glutBitmapCharacter(FONT, c);
		}
	}

	y1 = ty + ts; y2 = ty;
	glEnable(GL_TEXTURE_2D);

	tri->tex.glbind();

	glColor3ub(127, 127, 127);

	glBegin(GL_QUADS);
	glTexCoord2i(0, 0);
	glVertex2i(x1, y1);
	glTexCoord2i(0, tn);
	glVertex2i(x1, y2);
	glTexCoord2i(tn, tn);
	glVertex2i(x2, y2);
	glTexCoord2i(tn, 0);
	glVertex2i(x2, y1);
	glEnd();

	glColor3ub(255, 255, 255);

	float f = float(ts) / float(tn);

	glBegin(GL_TRIANGLES);
	tri->vtx[0].gltexture();
	glVertex2i(x1 + int(tri->vtx[0].u * f),
		   y1 - int(tri->vtx[0].v * f));
	tri->vtx[1].gltexture();
	glVertex2i(x1 + int(tri->vtx[1].u * f),
		   y1 - int(tri->vtx[1].v * f));
	tri->vtx[2].gltexture();
	glVertex2i(x1 + int(tri->vtx[2].u * f),
		   y1 - int(tri->vtx[2].v * f));
	glEnd();
}

void gui::select_texture_keys(int key, int, int)
{
	triangle *tri = map.selected_triangle();
	texture *tex = &tri->tex;
	float n;

	switch (key)
	{
	case '\e':
		cancel();
		break;
	case '\t':
		n = float(tn);
		for (unsigned i = 0; i < 3; i++)
		{
			tri->vtx[i].u = n - tri->vtx[i].u;
			tri->vtx[i].v = n - tri->vtx[i].v;
		}
		break;
	case 'F':
	case 'f':
		glutFullScreen();
		break;
	case 'T':
	case 't':
		tn = (tn > 1) ? 1 : 3;
		select_texture_calcsize();
		break;
	case -GLUT_KEY_LEFT:
		tex->first();
		break;
	case -GLUT_KEY_RIGHT:
		tex->last();
		break;
	case -GLUT_KEY_UP:
		if (!(tex->prev()))
			tex->first();
		break;
	case -GLUT_KEY_DOWN:
		if (!(tex->next()))
			tex->last();
		break;
	}

	tex->set_default();

	glutPostRedisplay();
}

void gui::select_texture_down(int x, int y)
{
	int u = ((x - tx) * tn * 64) / ts;
	int v = ((y - ty) * tn * 64) / ts;
	int i, a, b, n = 10;
	triangle *tri = map.selected_triangle();

	vtx = -1;

	for (i = 0; i < 3; i++)
	{
		a = abs(int(tri->vtx[i].u * 64.0) - u);
		b = abs(int(tri->vtx[i].v * 64.0) - v);
		a = a*a + b*b;
		if (a < n)
		{
			vtx = i;
			n = a;
		}
	}

	if (vtx >= 0)
	{
		print_texture(&tri->tex, &tri->vtx[vtx]);

		mouse_up_fn = &gui::select_texture_up;
		motion_fn = &gui::select_texture_motion;
	}
}

void gui::select_texture_up(int x, int y)
{
	select_texture_motion(x, y);
	str[0] = '\0';
	glutPostRedisplay();

	mouse_up_fn = 0;
	motion_fn = 0;
}

void gui::select_texture_motion(int x, int y)
{
	triangle *tri = map.selected_triangle();
	int n = tn * 64;
	int u = ((x - tx) * n) / ts;
	int v = ((y - ty) * n) / ts;

	if (u < 0) u = 0;
	if (u > n) u = n;

	if (v < 0) v = 0;
	if (v > n) v = n;

	tri->vtx[vtx].u = float(u) / 64.0;
	tri->vtx[vtx].v = float(v) / 64.0;
	print_texture(&tri->tex, &tri->vtx[vtx]);
}

void gui::print_texture(texture *tex, vertex *vtx)
{
	sprintf(str, "%.2f,%.2f",
		vtx->u * float(tex->width()),
		vtx->v * float(tex->height()));
	glutPostRedisplay();
}
