/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GUI_H__
#define __GUI_H__

#include <GL/glut.h>

#include "camera.h"
#include "landscape.h"

extern void screenshot(unsigned, unsigned);

class gui
{
 private:
	camera *cam;

	int menu;
	int file_menu;
	int block_menu;
	int tri_menu;

	char *filename;

	vector vec;
	triangle *new_tri;
	int vtx;
	int tx, ty, ts, tn;

	char str[24];
	square buf;

	void (gui::*draw_fn)();
	void (gui::*keyboard_fn)(int, int, int);
	void (gui::*mouse_down_fn)(int, int);
	void (gui::*mouse_up_fn)(int, int);
	void (gui::*motion_fn)(int, int);
 public:
	landscape map;
	int move_xy;

	gui();
	~gui();

	void init(camera *);
	void cancel();
	void set_menu(int);

	void draw_ui();

	void draw_map()
		{ vector v; cam->backward(&v); v.neg();
		  map.draw(&cam->box, &v); }

	void highlight_map()
		{ map.highlight(); }

	void load_map(char *file)
		{ map.load(filename = file); }

	void save_map()
		{ map.save(filename); }

	void keyboard(int key, int x, int y)
	{
		if (keyboard_fn) (this->*keyboard_fn)(key, x, y);
	}

	void special(int key, int x, int y)
	{
		if (keyboard_fn) (this->*keyboard_fn)(-key, x, y);
	}

	void mouse(int updown, int x, int y)
	{
		if (updown == GLUT_DOWN)
		{
			if (mouse_down_fn) (this->*mouse_down_fn)(x, y);
		}
		else
		{
			if (mouse_up_fn) (this->*mouse_up_fn)(x, y);
		}
	}

	void motion(int x, int y)
	{
		if (motion_fn) (this->*motion_fn)(x, y);
	}

	void select_block();
	void move_block();
	void rotate_block();
	void new_triangle();
	void select_triangle();
	void delete_triangle();
	void move_vertex();
	void select_texture();

	void copy();
	void cut();
	void paste();

	void screen_shot() { screenshot(unsigned(cam->get_width()),
					unsigned(cam->get_height())); }

 private:
	void print_height(int);
	void print_vertex(vector *);
	void print_triangle1(triangle *);
	void print_triangle2(triangle *);
	void print_triangle3(triangle *);
	void print_texture(texture *, vertex *);

	void edit_keys(int, int, int);

	void select_block_down(int, int);
	void select_block_up(int, int);

	void move_block_down(int, int);
	void move_block_up(int, int);
	void move_block_motion(int, int);

	void new_triangle_down1(int, int);
	void new_triangle_up1(int, int);
	void new_triangle_motion1(int, int);
	void new_triangle_down2(int, int);
	void new_triangle_up2(int, int);
	void new_triangle_motion2(int, int);
	void new_triangle_down3(int, int);
	void new_triangle_up3(int, int);
	void new_triangle_motion3(int, int);

	void select_triangle_down(int, int);
	void select_triangle_up(int, int);

	void move_vertex_down(int, int);
	void move_vertex_up(int, int);
	void move_vertex_motion(int, int);

	void select_texture_draw();
	void select_texture_keys(int, int, int);
	void select_texture_down(int, int);
	void select_texture_up(int, int);
	void select_texture_motion(int, int);
	void select_texture_calcsize();
};

#endif
