/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LANDSCAPE_H__
#define __LANDSCAPE_H__

#include "vector.h"
#include "bbox.h"
#include "texture.h"

class landscape;

class vertex : public vector
{
 public:
	GLfloat u, v;

	vertex();

	inline void gltexture() { glTexCoord2f(u, v); }

	int load(FILE *);
	int save(FILE *);
};

class triangle : public drawable
{
 public:
	triangle *next;
	vertex vtx[3];

 private:
	vector vec[3];

 public:
	triangle();

	void queue(landscape *, const vector *, const vector *);
	virtual void draw(landscape *);
	void highlight();

	int load(FILE *, texture *);
	int save(FILE *);

	int select_vertex(const vector *, const vector *, const vector *);
	float isect_line(const vector *, const vector *, const vector *);

	void copy(triangle *);
};

class square : public vector
{
 public:
	triangle *tris;

	square();
	~square();

	void draw(landscape *, const vector *);
	void highlight(landscape *);

	int load(FILE *, texture *);
	int save(FILE *);

	unsigned triangles();

	void set_height(float);
	void fix_vec(vector *);

	triangle *new_triangle(const vector *);
	void delete_triangle(triangle *);

	triangle *select_triangle(const vector *, const vector *);

	void copy(square *);
	void clear();
};

class landscape
{
 private:
	square *sel_sqr;
	triangle *sel_tri;
	drawable *first_d;
	square sqr[256][256];

 public:
	landscape();
	~landscape();

	square *selected_square() { return sel_sqr; }
	triangle *selected_triangle() { return sel_tri; }

	void draw(const bbox *, const vector *);
	void queue(drawable *);
	void highlight() { if (sel_sqr) sel_sqr->highlight(this); }

	int load(const char *);
	int save(const char *);

	square *select_square(const vector *, const vector *, const bbox *);
	triangle *select_next_triangle();
	triangle *select_triangle(const vector *org, const vector *dir)
		{ return sel_tri = sel_sqr->select_triangle(org, dir); }
	void unselect_square() { sel_sqr = 0; sel_tri = 0; }
	void unselect_triangle() { sel_tri = 0; }

	triangle *new_triangle(const vector *vec)
		{ return sel_tri = sel_sqr->new_triangle(vec); }
	void delete_triangle(triangle *tri)
		{ sel_sqr->delete_triangle(tri); sel_tri = 0; }
};

#endif
