/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <GL/glut.h>

#include "config.h"
#include "camera.h"
#include "landscape.h"
#include "gui.h"

static camera cam;
static texture tex;
static gui ui;

static int middle_button = GLUT_UP;
static int last_x = 0, last_y = 0;
static GLfloat sky[4] = { 0.0, 0.0, 0.0, 1.0 };

static void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_FOG);
	glEnable(GL_CULL_FACE);
	cam.perspective();

	cam.load();
	ui.draw_map();

	glDisable(GL_DEPTH_TEST);
	ui.highlight_map();

	glDisable(GL_FOG);
	glDisable(GL_CULL_FACE);
	cam.ortho();

	ui.draw_ui();

	glutSwapBuffers();
}

static void reshape(int width, int height)
{
	glViewport(0, 0, width, height);
	cam.set_size(width, height);
}

static void keyboard(unsigned char key, int x, int y)
{
	if (key == '\3') exit(0);
	ui.keyboard(key, x, y);
}

static void special(int key, int x, int y)
{
	ui.special(key, x, y);
}

static void mouse(int button, int updown, int x, int y)
{
	switch(button)
	{
	case GLUT_LEFT_BUTTON:
		ui.mouse(updown, x, y);
		break;
	case GLUT_MIDDLE_BUTTON:
		middle_button = updown;
		last_x = x; last_y = y;
		break;
	}
}

static void motion(int x, int y)
{
	if (middle_button == GLUT_DOWN)
	{
		int dx = x - last_x;
		int dy = y - last_y;

		if (dx || dy)
		{
			cam.pan -= float(dx) / 256.0;
			cam.tilt -= float(dy) / 256.0;
			glutWarpPointer(last_x, last_y);
			glutPostRedisplay();
		}
	}
	else 
	{
		ui.motion(x, y);
	}
}

static void init()
{
	cam.pos.set(2048.0, 2048.0, 16.0);
	cam.pan = 0.0;
	cam.tilt = 0.0;

	ui.init(&cam);

	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_START, NNEAR);
	glFogf(GL_FOG_END, FFAR);

	glClearColor(sky[0], sky[1], sky[2], sky[3]);
	glFogfv(GL_FOG_COLOR, sky);
}

int main(int argc, char **argv)
{
#ifdef GAME_DIR
	chdir(GAME_DIR);
#endif
	//glutInitWindowPosition(0, 0);
	glutInitWindowSize(WIDTH, HEIGHT);
	glutInit(&argc, argv);

	if (argc != 2)
	{
		fprintf(stderr, "Usage: %s filename\n", argv[0]);
		return 1;
	}

	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutCreateWindow("Map Editor");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(special);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutSetCursor(GLUT_CURSOR_CROSSHAIR);

	init();

	tex.init();
	ui.load_map(argv[1]);

	glutMainLoop();

	return 0;
}
