/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>

#include "matrix.h"

GLfloat matrix::identity[16] =
{
	1, 0, 0, 0,
	0, 1, 0, 0,
	0, 0, 1, 0,
	0, 0, 0, 1
};

//	x.x = m[0]; y.x = m[4]; z.x = m[8];
//	x.y = m[1]; y.y = m[5]; z.y = m[9];
//	x.z = m[2]; y.z = m[6]; z.z = m[10];

void matrix::set()
{
	for(unsigned i=0; i<16; i++)
		m[i] = identity[i];
}

void matrix::invert(matrix *p)
{
	vector v;
	v.set(-p->m[12], -p->m[13], -p->m[14]);
	m[0] = p->m[0]; m[1] = p->m[4]; m[2] = p->m[8]; m[3] = 0.0;
	m[4] = p->m[1]; m[5] = p->m[5]; m[6] = p->m[9]; m[7] = 0.0;
	m[8] = p->m[2]; m[9] = p->m[6]; m[10] = p->m[10]; m[11] = 0.0;
	m[12] = 0.0; m[13] = 0.0; m[14] = 0.0; m[15] = 1.0;
	translate(&v);
}

void matrix::translate(vector *v)
{
	m[12] += m[0] * v->v[0] + m[4] * v->v[1] + m[8] * v->v[2];
	m[13] += m[1] * v->v[0] + m[5] * v->v[1] + m[9] * v->v[2];
	m[14] += m[2] * v->v[0] + m[6] * v->v[1] + m[10] * v->v[2];
}

void matrix::rotate_x(float a)
{
	float s = sin(a * M_PI);
	float c = cos(a * M_PI);
	float i, j;

	i = m[4]; j = m[8];
	m[4] = (i * c) - (j * s);
	m[8] = (i * s) + (j * c);
	i = m[5]; j = m[9];
	m[5] = (i * c) - (j * s);
	m[9] = (i * s) + (j * c);
	i = m[6]; j = m[10];
	m[6] = (i * c) - (j * s);
	m[10] = (i * s) + (j * c);
}

void matrix::rotate_y(float a)
{
	float s = sin(a * M_PI);
	float c = cos(a * M_PI);
	float i, j;

	i = m[8]; j = m[0];
	m[8] = (i * c) - (j * s);
	m[0] = (i * s) + (j * c);
	i = m[9]; j = m[1];
	m[9] = (i * c) - (j * s);
	m[1] = (i * s) + (j * c);
	i = m[10]; j = m[2];
	m[10] = (i * c) - (j * s);
	m[2] = (i * s) + (j * c);
}

void matrix::rotate_z(float a)
{
	float s = sin(a * M_PI);
	float c = cos(a * M_PI);
	float i, j;

	i = m[0]; j = m[4];
	m[0] = (i * c) - (j * s);
	m[4] = (i * s) + (j * c);
	i = m[1]; j = m[5];
	m[1] = (i * c) - (j * s);
	m[5] = (i * s) + (j * c);
	i = m[2]; j = m[6];
	m[2] = (i * c) - (j * s);
	m[6] = (i * s) + (j * c);
}
