/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __TEXTURE_H__
#define __TEXTURE_H__

#include <stdio.h>
#include <GL/gl.h>

#include "list.h"

class drawable;

class texture_data : public node<texture_data>
{
 public:
	unsigned count;

 private:
	GLuint tex;
	int alpha;
	char *name;
	unsigned sizex, sizey;
	drawable *first_d;

  public:
	unsigned char index;

	texture_data();
	~texture_data();

	int load(const char *);

	inline const char *filename() { return name; }
	inline unsigned width() { return sizex; }
	inline unsigned height() { return sizey; }
	inline int has_alpha() { return alpha; }

	inline void glbind() { glBindTexture(GL_TEXTURE_2D, tex); }

	void queue(drawable *d);
	void draw(class landscape *);
};

class texture
{
 private:
	static list<texture_data> lh;
	static texture_data *def;
	texture_data *ptr;

	inline void set(texture_data *p)
	{
		if (ptr) ptr->count++;
		if ( (ptr = p) ) p->count--;
	}

 public:
	inline texture() { ptr = 0; }
	inline ~texture() { if (ptr) ptr->count--; }

	void init();
	int load(const char *);
	int load(FILE *);
	int save(FILE *);
	int find(const char *);

	inline const char *name() { return (ptr ? ptr->filename() : 0); }
	inline unsigned width() { return (ptr ? ptr->width() : 0); }
	inline unsigned height() { return (ptr ? ptr->height() : 0); }
	inline int has_alpha() { return (ptr ? ptr->has_alpha() : 0); }

	inline void glbind() { if (ptr) ptr->glbind(); }
	inline void queue(drawable *d) { if (ptr) ptr->queue(d); }

	inline void *first() { set(lh.first()); return ptr; }
	inline void *last() { set(lh.last()); return ptr; }
	inline void *next() { if (ptr) set(ptr->next()); return ptr; }
	inline void *prev() { if (ptr) set(ptr->prev()); return ptr; }

	inline void *get_default() { set(def); return ptr; }
	inline void set_default() { if(ptr) def = ptr; }

	inline void copy(texture *p) { set(p->ptr); }

	inline unsigned index() { return (ptr ? ptr->index : 0); }
	unsigned reindex();

	void draw(class landscape *);
};

class drawable
{
 public:
	drawable *next_d;
	texture tex;
	float dist;

	virtual ~drawable();
	virtual void draw(class landscape *);
};

inline void texture_data::queue(drawable *d)
{
	d->next_d = first_d;
	first_d = d;
}

#endif
