/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <float.h>

#include "vector.h"

/*
 *  intersection of a line and the line x=n,y=m.
 */

float vector::isect_xy_line(const vector *vec,
			    const vector *org, const vector *dir)
{
	float x = dir->v[0];
	float y = dir->v[1];
	float n = x*x + y*y; 

	if (fabs(n) < FLT_EPSILON) return 0;

	n = ((vec->v[0] - org->v[0]) * x +
	     (vec->v[1] - org->v[1]) * y) / n;

	v[0] = vec->v[0];
	v[1] = vec->v[1];
	v[2] = dir->v[2] * n + org->v[2];

	return n;
}

/* 
 *  intersection of a line and the plain z=n.
 */

float vector::isect_z_line(const vector *vec,
			   const vector *org, const vector *dir)
{
	float n = dir->v[2];

	if (fabs(n) < FLT_EPSILON) return 0;

	n = (vec->v[2] - org->v[2]) / n;

	v[0] = dir->v[0] * n + org->v[0];
	v[1] = dir->v[1] * n + org->v[1];
	v[2] = vec->v[2];

	return n;
}

/* 
 *  shortest vector between a point and a line.
 */

float vector::isect_point_line(const vector *pnt,
			       const vector *org, const vector *dir)
{
	vector v;
	float n;

	v.sub(pnt, org);
	n = v.dot(dir) / dir->val2();

	mul(dir, n);
	sub(&v);

	return n;
}

/*
 *
 */

float vector::isect_line_plane(const vector *l_org, const vector *l_dir,
			       const vector *p_org, const vector *p_norm)
{
	float n;

	n = p_norm->dot(l_dir);

	if (fabs(n) < FLT_EPSILON)
		return 0;

	n = (p_norm->dot(p_org) - p_norm->dot(l_org)) / n;

	mul(l_dir, n);
	add(l_org);

	return n;
}
