/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include <AL/alut.h>

#include "audio.h"
#include "config.h"
#include "misc.h"

list<source> audio::alh;
list<source> audio::flh;

int audio::s_ena = 0;


audio::audio(vector *p)
{
	src = 0;
	pos = p;
}

audio::~audio()
{
	if (src)
	{
		if (src->is_playing())
		{
			src->stop();
		}
		flh.add_head(src);
	}
}

void audio::init(int *argc, char **argv)
{
	if ( (s_ena = cfg.get_int("sound")) )
	{
		alutInit(argc, argv);
	}
}

void audio::update_all(camera *cam)
{
	if (!s_ena) return;

	vector v;
	float o[6];

	cam->backward(&v);
	o[0] = -v.x();
	o[1] = -v.y();
	o[2] = -v.z();
	cam->upward(&v);
	o[3] = v.x();
	o[4] = v.y();
	o[5] = v.z();

	cam->pos.allistener();
	alListenerfv(AL_ORIENTATION, o);

	source *s, *sn = alh.first();

	while ( (s = sn) )
	{
		sn = s->next();

		s->update();
	}
}

void audio::update()
{
	if (src->is_playing())
	{
		src->set_position(pos);
	}
	else
	{
		flh.add_head(src);		
	}
}

sound *audio::load_sound(const char *file)
{
	if (!s_ena)
		return 0;

	char fname[1024];
	ALvoid *wav = 0;
	ALsizei fmt;
	ALsizei size;
	ALsizei bits;
	ALsizei freq;

	sprintf(fname, "sounds/%s", file);
	if ( !alutLoadWAV(fname, &wav, &fmt, &size, &bits, &freq) )
		return 0;

	sound *snd = new sound(wav, fmt, size, freq);

	alutUnloadWAV(fmt, wav, size, freq);

	return snd;
}

void audio::play_sound(gstring *str, int loop, float gain, float pri)
{
	sound *snd;

	if ( !(snd = str->snd()) )
		return;

	if (!src)
	{
		if ( !(src = flh.first()) )
		{
			src = new source;
		}
		src->aud = this;
		alh.add_tail(src);
	}
	else if (src->is_playing())
	{
		if (src->pri > pri)
			return;
		src->stop();
	}

	src->set_sound(snd);
	src->set_gain(gain);
	src->set_looping(loop);
	src->set_position(pos);
	src->play();

	src->pri = pri;
}

void audio::stop_sound()
{
	if (src)
	{
		src->stop();
		flh.add_head(src);		
	}
}


sound::sound(ALvoid *wav, ALsizei fmt, ALsizei size, ALsizei freq)
{
	alGenBuffers(1, &buf);
	alBufferData(buf, fmt, wav, size, freq);
}

sound::~sound()
{
	alDeleteBuffers(1, &buf);
}


source::source() : node<source>(this)
{
	alGenSources(1, &sid);
}

source::~source()
{
	alDeleteSources(1, &sid);
}
