/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __AUDIO_H__
#define __AUDIO_H__

#include <AL/al.h>

#include "camera.h"
#include "vector.h"
#include "list.h"
#include "gstring.h"

class sound;
class source;

class audio
{
 private:
	source *src;
	vector *pos;

	static list<source> alh;
	static list<source> flh;

	static int s_ena;

 public:
	audio(vector *);
	~audio();

	static void init(int *, char **);
	static void update_all(camera *);

	static sound *load_sound(const char *);
	void play_sound(gstring *, int, float, float);
	void stop_sound();
	void update();
};

inline void audio_init(int *argc, char **argv)
{
	audio::init(argc, argv);
}

inline void audio_update(camera *cam)
{
	audio::update_all(cam);
}

inline sound *load_sound(const char *file)
{
	return audio::load_sound(file);
}

class sound
{
 private:
	ALuint buf;

 public:
	sound(ALvoid *, ALsizei, ALsizei, ALsizei);
	~sound();

	ALuint albuffer() { return buf; }
};

class source : public node<source>
{
 private:
	ALuint sid;

 public:
	audio *aud;
	float pri;

	source();
	~source();

	void set_sound(sound *snd)
		{ alSourcei(sid, AL_BUFFER, snd->albuffer()); }
	void set_looping(ALint loop)
		{ alSourcei(sid, AL_LOOPING, loop); }
	void set_gain(ALfloat gain)
		{ alSourcef(sid, AL_GAIN, gain); }
	void set_position(vector *pos)
		{ pos->alsource(sid); }

	void play() { alSourcePlay(sid); }
	void stop() { alSourceStop(sid); }

	void update() { aud->update(); }

	int is_playing()
	{
		ALint state;
		alGetSourcei(sid, AL_SOURCE_STATE, &state);
		return (state == AL_PLAYING);
	}
};

#endif
