/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "bbox.h"

int bbox::intersect(const bbox *box) const
{
	if (min.x() > box->max.x())
		return 0;
	if (max.x() < box->min.x())
		return 0;

	if (min.y() > box->max.y())
		return 0;
	if (max.y() < box->min.y())
		return 0;

	if (min.z() > box->max.z())
		return 0;
	if (max.z() < box->min.z())
		return 0;

	return 1;
}

int bbox::intersect(const vector *l0, const vector *l1) const
{
	vector v, v1, v2;
	float tn = 0.0;
	float tf = 1.0;

	v.sub(l1, l0);
	v1.sub(&min, l0);
	v2.sub(&max, l0);

	if (v.x())
	{
		float t1 = v1.x() / v.x();
		float t2 = v2.x() / v.x();

		if (t1 > t2)
		{
			float t = t2;
			t2 = t1;
			t1 = t;
		}

		if (t1 > tn)
			tn = t1;
		if (t2 < tf)
			tf = t2;
		if (tn > tf)
			return 0;
	}
	else
	{
		if (v1.x() > 0.0)
			return 0;
		if (v2.x() < 0.0)
			return 0;
	}

	if (v.y())
	{
		float t1 = v1.y() / v.y();
		float t2 = v2.y() / v.y();

		if (t1 > t2)
		{
			float t = t2;
			t2 = t1;
			t1 = t;
		}

		if (t1 > tn)
			tn = t1;
		if (t2 < tf)
			tf = t2;
		if (tn > tf)
			return 0;
	}
	else
	{
		if (v1.y() > 0.0)
			return 0;
		if (v2.y() < 0.0)
			return 0;
	}


	if (v.z())
	{
		float t1 = v1.z() / v.z();
		float t2 = v2.z() / v.z();

		if (t1 > t2)
		{
			float t = t2;
			t2 = t1;
			t1 = t;
		}

		if (t1 > tn)
			tn = t1;
		if (t2 < tf)
			tf = t2;
		if (tn > tf)
			return 0;
	}
	else
	{
		if (v1.z() > 0.0)
			return 0;
		if (v2.z() < 0.0)
			return 0;
	}

	return 1;
}
