/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gstring.h"
#include "exec.h"
#include "builtins.h"
#include "entity.h"
#include "landscape.h"
#include "sky.h"
#include "audio.h"
#include "misc.h"

fn_new::fn_new(list<function> *lh) : function(lh)
{
	init("new", 1);
}

void fn_new::call(unsigned ent)
{
	variable *v = vars();
	v[1].idx = (new entity)->idx();
}

fn_delete::fn_delete(list<function> *lh) : function(lh)
{
	init("new", 0);
}

void fn_delete::call(unsigned ent)
{
	entity_array()[ent] = 0;
}

fn_player::fn_player(list<function> *lh) : function(lh)
{
	init("player", 1);
}

void fn_player::call(unsigned ent)
{
	variable *v = vars();
	v[1].idx = player_entity()->idx();
}

fn_camera::fn_camera(list<function> *lh) : function(lh)
{
	init("camera", 1);
}

void fn_camera::call(unsigned ent)
{
	variable *v = vars();
	v[1].idx = camera_entity()->idx();
}

fn_origin::fn_origin(list<function> *lh) : function(lh)
{
	init("origin", 3);
}

void fn_origin::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		v[1].flt = e->origin.x();
		v[2].flt = e->origin.y();
		v[3].flt = e->origin.z();
	}
	else
	{
		v[1].flt = 0;
		v[2].flt = 0;
		v[3].flt = 0;
	}
}

fn_angles::fn_angles(list<function> *lh) : function(lh)
{
	init("angles", 3);
}

void fn_angles::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		v[1].flt = e->angles.x();
		v[2].flt = e->angles.y();
		v[3].flt = e->angles.z();
	}
	else
	{
		v[1].flt = 0;
		v[2].flt = 0;
		v[3].flt = 0;
	}
}

fn_velocity::fn_velocity(list<function> *lh) : function(lh)
{
	init("velocity", 3);
}

void fn_velocity::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		v[1].flt = e->velocity.x();
		v[2].flt = e->velocity.y();
		v[3].flt = e->velocity.z();
	}
	else
	{
		v[1].flt = 0;
		v[2].flt = 0;
		v[3].flt = 0;
	}
}

fn_avelocity::fn_avelocity(list<function> *lh) : function(lh)
{
	init("avelocity", 3);
}

void fn_avelocity::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		v[1].flt = e->avelocity.x();
		v[2].flt = e->avelocity.y();
		v[3].flt = e->avelocity.z();
	}
	else
	{
		v[1].flt = 0;
		v[2].flt = 0;
		v[3].flt = 0;
	}
}

fn_setorigin::fn_setorigin(list<function> *lh) : function(lh)
{
	init("setorigin", 3);
}

void fn_setorigin::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->origin.set(v[1].flt, v[2].flt, v[3].flt);
	}
}

fn_setangles::fn_setangles(list<function> *lh) : function(lh)
{
	init("setangles", 3);
}

void fn_setangles::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->angles.set(v[1].flt, v[2].flt, v[3].flt);
	}
}


fn_setvelocity::fn_setvelocity(list<function> *lh) : function(lh)
{
	init("setvelocity", 3);
}

void fn_setvelocity::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->velocity.set(v[1].flt, v[2].flt, v[3].flt);
	}
}

fn_setavelocity::fn_setavelocity(list<function> *lh) : function(lh)
{
	init("setavelocity", 3);
}

void fn_setavelocity::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->avelocity.set(v[1].flt, v[2].flt, v[3].flt);
	}
}

fn_setsize::fn_setsize(list<function> *lh) : function(lh)
{
	init("setsize", 6);
}

void fn_setsize::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->size.min.set(v[1].flt, v[2].flt, v[3].flt);
		e->size.max.set(v[4].flt, v[5].flt, v[6].flt);
	}
}

fn_setmodel::fn_setmodel(list<function> *lh) : function(lh)
{
	init("setmodel", 1);
}

void fn_setmodel::call(unsigned ent)
{
	entity *e;

	if ((e = entity_array()[ent]))
	{
		e->set_mdl(string_array()[vars()[1].idx]);
		e->reset_frame();
	}
}

fn_setframe::fn_setframe(list<function> *lh) : function(lh)
{
	init("setframe", 1);
}

void fn_setframe::call(unsigned ent)
{
	entity *e;

	if ((e = entity_array()[ent]))
	{
		e->set_frame(unsigned(vars()[1].flt));
	}
}

fn_waterheight::fn_waterheight(list<function> *lh) : function(lh)
{
	init("waterheight", 1);
}

void fn_waterheight::call(unsigned ent)
{
	vars()[1].flt = map.water_height();
}

fn_setwatertexture::fn_setwatertexture(list<function> *lh) : function(lh)
{
	init("setwatertexture", 1);
}

void fn_setwatertexture::call(unsigned ent)
{
	map.set_water_texture(string_array()[vars()[1].idx]);
}

fn_setwaterheight::fn_setwaterheight(list<function> *lh) : function(lh)
{
	init("setwaterheight", 1);
}

void fn_setwaterheight::call(unsigned ent)
{
	map.set_water_height(vars()[1].flt);
}

fn_setclouds::fn_setclouds(list<function> *lh) : function(lh)
{
	init("setclouds", 3);
}

void fn_setclouds::call(unsigned ent)
{
	variable *v = vars();
	sky.set_clouds(string_array()[v[1].idx], v[2].flt, v[3].flt);
}

fn_setredsun::fn_setredsun(list<function> *lh) : function(lh)
{
	init("setredsun", 2);
}

void fn_setredsun::call(unsigned ent)
{
	variable *v = vars();
	sky.set_red(v[1].flt, v[2].flt);
}

fn_setbluesun::fn_setbluesun(list<function> *lh) : function(lh)
{
	init("setbluesun", 2);
}

void fn_setbluesun::call(unsigned ent)
{
	variable *v = vars();
	sky.set_blue(v[1].flt, v[2].flt);
}

static float slope;

fn_hota::fn_hota(list<function> *lh) : function(lh)
{
	init("hota", 4);
}

void fn_hota::call(unsigned ent)
{
	vector vec;
	variable *v = vars();

	vec.set(v[2].flt, v[3].flt, v[4].flt);
	v[1].flt = map.height_above(&slope, &vec);
}

fn_hotb::fn_hotb(list<function> *lh) : function(lh)
{
	init("hotb", 4);
}

void fn_hotb::call(unsigned ent)
{
	vector vec;
	variable *v = vars();

	vec.set(v[2].flt, v[3].flt, v[4].flt);
	v[1].flt = map.height_below(&slope, &vec);
}

fn_gradient::fn_gradient(list<function> *lh) : function(lh)
{
	init("gradient", 1);
}

void fn_gradient::call(unsigned ent)
{
	float f = sqrt(1.0 - slope*slope);

	vars()[1].flt = f / slope;
}

fn_intersect::fn_intersect(list<function> *lh) : function(lh)
{
	init("intersect", 3);
}

void fn_intersect::call(unsigned ent)
{
	entity *e;
	vector vec;
	bbox box;
	variable *v = vars();
	function *fn = string_array()[v[2].idx]->fn();
	float f = v[3].flt;
	float ret = 0.0;

	if ((e = entity_array()[ent]))
	{
		if (fn)
		{
			if ((ret = e->intersect(fn)))
				goto done;
		}

		if (f)
		{
			e->position(&vec, 1.0); 
			box.add(&e->size, &vec);

			if (map.intersect(&box))
				ret = f;
		}
	}
 done:
	v[1].flt = ret;
}

fn_keys::fn_keys(list<function> *lh) : function(lh)
{
	init("keys", 1);
}

void fn_keys::call(unsigned ent)
{
	variable *v = vars();

	v[1].flt = keys;
}

static matrix m;

fn_makevectors::fn_makevectors(list<function> *lh) : function(lh)
{
	init("makevectors", 3);
}

void fn_makevectors::call(unsigned ent)
{
	vector p, a;
	variable *v = vars();

	p.set(0.0, 0.0, 0.0);
	a.set(v[1].flt, v[2].flt, v[3].flt);
	m.set(&p, &a);
}

fn_v_forward::fn_v_forward(list<function> *lh) : function(lh)
{
	init("v_forward", 3);
}

void fn_v_forward::call(unsigned ent)
{
	vector vec;
	variable *v = vars();

	vec.get_x(&m);
	v[1].flt = vec.x();
	v[2].flt = vec.y();
	v[3].flt = vec.z();
}

fn_v_left::fn_v_left(list<function> *lh) : function(lh)
{
	init("v_left", 3);
}

void fn_v_left::call(unsigned ent)
{
	variable *v = vars();
	vector vec;

	vec.get_y(&m);
	v[1].flt = vec.x();
	v[2].flt = vec.y();
	v[3].flt = vec.z();
}

fn_v_up::fn_v_up(list<function> *lh) : function(lh)
{
	init("v_up", 3);
}

void fn_v_up::call(unsigned ent)
{
	vector vec;
	variable *v = vars();

	vec.get_z(&m);
	v[1].flt = vec.x();
	v[2].flt = vec.y();
	v[3].flt = vec.z();
}

fn_normalize::fn_normalize(list<function> *lh) : function(lh)
{
	init("normalize", 6);
}

void fn_normalize::call(unsigned ent)
{
	vector vec;
	variable *v = vars();

	vec.set(v[4].flt, v[5].flt, v[6].flt);
	vec.unit();
	v[1].flt = vec.x();
	v[2].flt = vec.y();
	v[3].flt = vec.z();
}

fn_precache_sound::fn_precache_sound(list<function> *lh) : function(lh)
{
	init("precache_sound", 1);
}

void fn_precache_sound::call(unsigned ent)
{
	(string_array()[vars()[1].idx])->snd();
}

fn_play_sound::fn_play_sound(list<function> *lh) : function(lh)
{
	init("play_sound", 3);
}

void fn_play_sound::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->aud.play_sound(string_array()[v[1].idx],
				  AL_FALSE, v[2].flt, v[3].flt);
	}
}

fn_play_sound_loop::fn_play_sound_loop(list<function> *lh) : function(lh)
{
	init("play_sound_loop", 3);
}

void fn_play_sound_loop::call(unsigned ent)
{
	entity *e;
	variable *v = vars();

	if ((e = entity_array()[ent]))
	{
		e->aud.play_sound(string_array()[v[1].idx],
				  AL_TRUE, v[2].flt, v[3].flt);
	}
}

fn_stop_sound::fn_stop_sound(list<function> *lh) : function(lh)
{
	init("stop_sound", 0);
}

void fn_stop_sound::call(unsigned ent)
{
	entity *e;

	if ((e = entity_array()[ent]))
	{
		e->aud.stop_sound();
	}
}
