/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CAMERA_H__
#define __CAMERA_H__

#include "matrix.h"
#include "bbox.h"

class camera
{
 private:
	float width, height, fov;
	vector nearv, farv;
	matrix wtoc, ctow;
	float hx, hz, vy, vz;

 public:
	vector pos, tgt;
	bbox box;

	void init();

	void perspective();
	void ortho();

	void make();
	void load();

	void set_size(float, float);
	void set_fov(float);

	int check_bsphere(vector *, float);

	inline float get_width() { return width; }
	inline float get_height() { return height; }

	inline float get_near() { return nearv.z(); }
	inline float get_far() { return farv.z(); }

	inline float o_width() { return 24.0 * width / height; }
	inline float o_height() { return 24.0; }

	inline void sideways(vector *v)
		{ v->get_x(&ctow); }
	inline void upward(vector *v)
		{ v->get_y(&ctow); }
	inline void backward(vector *v)
		{ v->get_z(&ctow); }
	inline void position(vector *v)
		{ v->get_w(&ctow); }

 private:
	void min_max(vector *);
};

#endif
