/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "config.h"

config::config()
{
	char tmp[1024];
	const char *home;
	FILE *fp;

	clean = 1;

	set_float("width", 640.0);
	set_float("height", 480.0);
	set_float("near", 0.5);
	set_float("far", 256.0);
	set_float("fov", 1.0);
	set_int("mipmap", 1);
	set_float("lod_bias", 0.0);
	set_int("filter", 1);
	set_int("sound", 1);

//	set_string("game_dir", "/usr/games/lost");

	if ( (home = getenv("HOME")) )
	{
		sprintf(tmp, "%s/.lost", home);
	}
	else
	{
		strcpy(tmp, ".lost");
	}

	set_string("dot_dir", tmp);
	strcat(tmp, "/config");

	if ( (fp = fopen(tmp, "r")) )
	{
		while (fgets(tmp, 1024, fp))
		{
			char *name, *value, *end;

			if ( (name = strtok(tmp, " \t")) &&
			     (value = strtok(0, "\n")) )
			{
				if (value[0] == '"')
				{
					if ( (end = strrchr(++value, '"')) )
					{
						*end = 0;
						set_string(name, value);
					}
				}
				else
				{
					end = 0;
					float f = strtod(value, &end);

					if (end != value)
					{
						set_float(name, f);
					}
				}
			}
		}
	
		fclose(fp);

		clean = 1;
	}

	if ( (home = get_string("dot_dir")) &&
	     access(home, F_OK) )
	{
		mkdir(home, 0777);
	}
}

config::~config()
{
	char tmp[1024];
	const char *dot;
	FILE *fp;

	if (clean) return;

	if ( !(dot = get_string("dot_dir")) )
		return;

	sprintf(tmp, "%s/config", dot);

	if (! (fp = fopen(tmp, "w")) )
		return;

	for (fvar *fv = fvars.first(); fv; fv = fv->next())
	{
		fprintf(fp, "%s %.9g\n", fv->name(), fv->value());
	}

	for (svar *sv = svars.first(); sv; sv = sv->next())
	{
		fprintf(fp, "%s \"%s\"\n", sv->name(), sv->value());
	}
	
	fclose(fp);
}

void config::set_float(const char *name, float value)
{
	for (fvar *var = fvars.first(); var; var = var->next())
	{
		if (!strcmp(var->name(), name))
		{
			if (var->value() != value)
			{
				var->set(value);
				clean = 0;
			}
			return;
		}
	}

	fvars.add_tail(new fvar(name, value));
	clean = 0;
}

float config::get_float(const char *name)
{
	for (fvar *var = fvars.first(); var; var = var->next())
	{
		if (!strcmp(var->name(), name))
		{
			return var->value();
		}
	}

	return 0.0;
}

void config::set_string(const char *name, const char *value)
{
	for (svar *var = svars.first(); var; var = var->next())
	{
		if (!strcmp(var->name(), name))
		{
			if (var->value() != value)
			{
				var->set(value);
				clean = 0;
			}
			return;
		}
	}

	svars.add_tail(new svar(name, value));
	clean = 0;
}

const char *config::get_string(const char *name)
{
	for (svar *var = svars.first(); var; var = var->next())
	{
		if (!strcmp(var->name(), name))
		{
			return var->value();
		}
	}

	return 0;
}

fvar::fvar(const char *n, float v) : node<fvar>(this)
{
	strcpy(nam = new char[strlen(n) + 1], n);
	val = v;
}

fvar::~fvar()
{
	delete nam;
}

svar::svar(const char *n, const char *v) : node<svar>(this)
{
	strcpy(nam = new char[strlen(n) + 1], n);
	strcpy(val = new char[strlen(v) + 1], v);
}

svar::~svar()
{
	delete nam;
	delete val;
}

void svar::set(const char *v)
{
	delete val;
	strcpy(val = new char[strlen(v) + 1], v);
}
