/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CONFIG_H__
#define __CONFIG_H__

#include "list.h"

class fvar: public node<fvar>
{
 private:
	char *nam;
	float val;
	
 public:
	fvar(const char *, float);
	~fvar();

	inline const char *name() { return nam; };
	inline float value() { return val; };
	inline void set(float f) { val = f; };
};

class svar: public node<svar>
{
 private:
	char *nam;
	char *val;

 public:
	svar(const char *, const char *);
	~svar();

	void set(const char *);

	inline const char *name() { return nam; };
	inline const char *value() { return val; };
};

class config
{
 private:
	int clean;

	list<fvar> fvars;
	list<svar> svars;
	
 public:
	config();
	~config();

	void set_float(const char *, float);
	void set_string(const char *, const char *);

	float get_float(const char *);
	const char *get_string(const char *);

	inline void set_int(const char *var, int i) { set_float(var, float(i)); }
	inline int get_int(const char *var) { return int(get_float(var)); }
};

#endif
