/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ENTITY_H__
#define __ENTITY_H__

#include "gstring.h"
#include "model.h"
#include "audio.h"
#include "bbox.h"
#include "matrix.h"
#include "list.h"

class entity : private node<entity>
{
 private:
	model *mdl;
	union variable *varp;
	unsigned char k0, k1;
	unsigned short id;

	static entity *array[65536];
	static list<entity> lh;
	static unsigned last_id;

 public:
	vector origin, velocity;
	vector angles, avelocity;
	bbox size;
	audio aud;

	entity(unsigned i=0);
	~entity();

	inline union variable *vars() { return varp; }
	inline unsigned idx() { return id; }
	inline void set_mdl(gstring *s) { mdl = s->mdl(); }
	inline void reset_frame() { k0 = k1 = 0; }
	inline void set_frame(unsigned k)
		{ if (mdl && k < mdl->frames()) k1 = k; }
	inline void position(vector *v, float t)
		{ v->mul(&velocity, t); v->add(&origin); }
	inline void rotation(vector *v, float t)
		{ v->mul(&avelocity, t); v->add(&angles); }
	float intersect(class function *);

	static inline entity **entity_array() { return array; }
	static void draw(class camera *, float);
	static void tick();
	static void delete_all();
	static entity *get_ent(unsigned);
};

inline entity **entity_array()
{
	return entity::entity_array();
}

inline void draw_entities(class camera *cam, float t)
{
	entity::draw(cam, t);
}

inline void tick_entities()
{
	entity::tick();
}

inline void delete_entities()
{
	entity::delete_all();
}

inline entity *player_entity()
{
	return entity::get_ent(1);
}

inline entity *camera_entity()
{
	return entity::get_ent(2);
}

#endif
