/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>

#include "gstring.h"
#include "font.h"

font::font()
{
	chars = 0;
}

font::~font()
{
	if (chars) delete [] chars;
}

int font::load(const char *file)
{
	FILE *fp;
	gstring *gstr;
	char fname[1024];
	unsigned n;

	sprintf(fname, "fonts/%s", file);

	if ( !(fp = fopen(fname, "rb")) )
		return -1;

	if ( fread(fname, 1, 4, fp) != 4 ||
	     strncmp(fname, "CJPF", 4) )
	{
		fclose(fp);
		return -1;
	}

	if (!(gstr = load_string(fp)))
	{
		fclose(fp);
		return -1;
	}

	if (!(tex = gstr->tex()))
	{
		fclose(fp);
		return -1;
	}

	if ( (first = fgetc(fp)) > 255 )
	{
		fclose(fp);
		return -1;
	}

	if ( (last = fgetc(fp)) > 255 )
	{
		fclose(fp);
		return -1;
	}

	if (last < first)
	{
		fclose(fp);
		return -1;
	}

	if ( (n = fgetc(fp)) > 255 )
	{
		fclose(fp);
		return -1;
	}
	h = float(n) / 256.0;

	if (chars) delete [] chars;
	chars = new font_char[last + 1 - first];

	for (unsigned i = first; i <= last; i++)
	{
		font_char *fc = &chars[i - first];

		if ( (n = fgetc(fp)) > 255 )
		{
			fclose(fp);
			return -1;
		}
		fc->u = float(n) / 256.0;

		if ( (n = fgetc(fp)) > 255 )
		{
			fclose(fp);
			return -1;
		}
		fc->v = float(n) / 256.0;

		if ( (n = fgetc(fp)) > 255 )
		{
			fclose(fp);
			return -1;
		}
		fc->w = float(n) / 256.0;
	}

	fclose(fp);

	return 0;
}

float font::draw(const char *str, float x, float y, float size)
{
	unsigned c;
	float i = x, j = y + size;

	glBegin(GL_QUADS);

	while ( (c = (unsigned char)(*(str++))) )
	{
		if (c >= first && c <= last)
		{
			font_char *fc = &chars[c - first];
			float t = fc->w * size / h;

			glTexCoord2f(fc->u, fc->v);
			glVertex2f(i, y);

			float v = fc->v + h;

			glTexCoord2f(fc->u, v);
			glVertex2f(i, j);

			float u = fc->u + fc->w;
			i += t;

			glTexCoord2f(u, v);
			glVertex2f(i, j);

			glTexCoord2f(u, fc->v);
			glVertex2f(i, y);
		}
	}

	glEnd();

	return i - x;
}

float font::width(const char *str)
{
	unsigned c;
	float w = 0.0;

	while ( (c = (unsigned char)(*(str++))) )
	{
		if (c >= first && c <= last)
		{
			w += chars[c - first].w;
		}
	}

	return w / h;
}
