/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>

#include "texture.h"
#include "model.h"
#include "audio.h"
#include "gstring.h"
#include "exec.h"

gstrings gstrings::instance;

gstrings::gstrings()
{
	for (unsigned i = 0; i < 256; i++)
	{
		hash[i] = 0;
	}

	new_string("");
}

gstring *gstrings::new_string(const char *cstr)
{
	gstring *str, **ptr = &hash[(unsigned char)cstr[0]];

	while ((str = *ptr))
	{
		if (!strncmp(str->cstr, cstr, 255))
			return str;
		ptr = &str->next;
	}

	str = new gstring(cstr);
	*ptr = str;
	array[str->idx()] = str;

	return str;
}

gstring *gstrings::load_string(FILE *fp)
{
	unsigned i;
	char cstr[256];

	if ( (i = fgetc(fp)) > 255 )
		return 0;
	if (i)
	{
		if (fread(cstr, 1, i, fp) != i)
			return 0;
	}
	cstr[i] = '\0';

	return new_string(cstr);
}

void gstrings::reset_variables()
{
	for (unsigned i = 0; i < gstring::nstrs; i++)
	{
		array[i]->vtype = 0;
		array[i]->vidx = 0;
		array[i]->fn_p = 0;
	}
	gstring::nvars = 0;
}

unsigned gstring::nstrs = 0;
unsigned gstring::nvars = 0;

gstring::gstring(const char *str)
{
	strcpy(cstr = new char[strlen(str) + 1], str);  // cstr = strdup(str);
	next = 0;
	sidx = nstrs++;
	vtype = 0;
	vidx = 0;
	tex_p = 0;
	mdl_p = 0;
	snd_p = 0;
	fn_p = 0;
}

texture *gstring::tex()
{
	texture *t;

	if (tex_p) return tex_p;

	if (!sidx) return 0;

	t = new texture;

	if (t->load(cstr))
	{
		delete t;
		return 0;
	}

	return (tex_p = t);
}

model *gstring::mdl()
{
	model *m;

	if (mdl_p) return mdl_p;

	if (!sidx) return 0;

	m = new model;

	if (m->load(cstr))
	{
		delete m;
		return 0;
	}
		
	return (mdl_p = m);
}

sound *gstring::snd()
{
	if (snd_p) return snd_p;

	if (!sidx) return 0;

	snd_p = load_sound(cstr);

	return snd_p;
}

void gstring::set_fn(function *f)
{
	if (fn_p && fn_p != f)
		delete fn_p;
	fn_p = f;
}

int gstring::set_var_type(unsigned char t)
{
	if (vtype) return (vtype != t);
	
	vtype = t;
	vidx = nvars;

	nvars += ((t == T_VECTOR) ? 3 : 1);

	return (nvars > 256);
}
