/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GSTRING_H__
#define __GSTRING_H__

#include <stdio.h>

class gstring
{
	friend class gstrings;

 private:
	gstring *next;
	char *cstr;
	unsigned short sidx;
	unsigned char vtype;
	unsigned char vidx;
	class texture *tex_p;
	class model *mdl_p;
	class sound *snd_p;
	class function *fn_p;

	static unsigned nstrs;
	static unsigned nvars;

	gstring(const char *);

 public:

	inline const char *val() { return cstr; }
	inline unsigned idx() { return sidx; }

	class texture *tex();
	class model *mdl();
	class sound *snd();

	inline class function *fn() { return fn_p; }
	void set_fn(class function *f);

	inline unsigned char var_type() { return vtype; }
	inline unsigned var_index() { return vidx; }
	int set_var_type(unsigned char);

	inline static unsigned num_vars() { return nvars; }
};

class gstrings
{
 private:
	gstring *hash[256];
	gstring *array[65536];

	gstrings();

 public:
	static gstrings instance;

	inline gstring **string_array() { return array; }

	gstring *new_string(const char *);
	gstring *load_string(FILE *);

	void reset_variables();
};

inline gstring *new_string(const char *str)
{
	return gstrings::instance.new_string(str);
}

inline gstring *load_string(FILE *fp)
{
	return gstrings::instance.load_string(fp);
}

inline void reset_variables()
{
	gstrings::instance.reset_variables();
}

inline gstring **string_array()
{
	return gstrings::instance.string_array();
}

#endif
