/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LANDSCAPE_H__
#define __LANDSCAPE_H__

#include "vector.h"
#include "gstring.h"
#include "bbox.h"
#include "texture.h"
#include "camera.h"

class l_triangle;
class landscape;

class l_texture
{
 public:
	l_triangle *first;
	texture *tex;

	l_texture();
	~l_texture();

	int load(FILE *);
	void draw();

	inline void glbind() { tex->glbind(); }
	inline int has_alpha() { return tex->has_alpha(); }
};

class l_vertex : public vector
{
 private:
	float u, v;

 public:
	inline void gltexture() { glTexCoord2f(u, v); }

	int load(FILE *, const vector *);
};

class l_triangle
{
 public:
	l_triangle *next;
	float dist;

 private:
	l_vertex vtx[3];
	vector norm;
	l_texture *tex;

 public:
	void queue(landscape *, camera *);
	void draw();
	int load(FILE *, const vector *, l_texture *);
	void make_normal();
	void min_max(bbox *);
	void radius(const vector *, float *);

	void height_above(float *, float *, const vector *);
	void height_below(float *, float *, const vector *);
	int intersect(const bbox *);
	int intersect(const vector *, const vector *);

	inline void glbind() { tex->glbind(); }
};

class l_square
{
 private:
	unsigned char ntri;
	l_triangle *tris;
	vector center;
	float radius;
	float min_z, max_z;

 public:
	l_square();
	~l_square();

	void draw(landscape *, camera *);
	int load(FILE *, vector *, l_texture *);
	void make_bsphere();

	float height_above(float *, const vector *);
	float height_below(float *, const vector *);
	int intersect(const bbox *);
};

class landscape
{
 private:
	texture *wtex;
	float wz;

	l_triangle *above;
	l_triangle *below;

	unsigned ntex;
	l_texture tex[256];
	l_square sqr[256][256];

 public:
	landscape();
	~landscape();

	void draw(camera *);
	void queue(l_triangle *, float);
	void draw_water(camera *);
	int load(const char *);

	float height_above(float *, const vector *);
	float height_below(float *, const vector *);
	int intersect(const bbox *);

	void set_water_texture(gstring *str) { wtex = str->tex(); }
	void set_water_height(float z) { wz = z; }
	float water_height() { return wz; }
};

#endif
