/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LIST_H__
#define __LIST_H__

// This is a circular linked list with dummy head/tail node.

template <class Type> class list;

template <class Type> class node	// A node in a list.
{
	friend class list<Type>;	// Allo a list to manipulate nodes.

 private:
	node<Type> *ln_next, *ln_prev;
	Type *ln_data;

	void unlink()
	{
		node<Type> *nln = ln_next, *pln = ln_prev;

		nln->ln_prev = pln;
		pln->ln_next = nln;
	}

 public:
	node(Type *ptr) { ln_data = ptr; ln_next = ln_prev = this; }
	~node() { unlink(); }

	Type *next() { return ln_next->ln_data; }
	Type *prev() { return ln_prev->ln_data; }

	void remove()	// Remove ourself from the list we are in.
	{
		unlink();
		ln_next = this;
		ln_prev = this;
	}
};

template <class Type> class list : private node<Type>	// A list is a node.
{
 public:
	list() : node<Type>(0) { }

	Type *first() { return ln_next->ln_data; }
	Type *last() { return ln_prev->ln_data; }

	void add_head(node<Type> *ln)	// Add a node to the start
	{				// of this list.
		node<Type> *nln;

		ln->unlink();
		nln = ln_next;

		ln_next = ln;
		ln->ln_prev = this;

		ln->ln_next = nln;
		nln->ln_prev = ln;
	}

	void add_tail(node<Type> *ln)	// Add a node to the end
	{				// of this list.
		node<Type> *pln;

		ln->unlink();
		pln = ln_prev;

		ln_prev = ln;
		ln->ln_next = this;

		ln->ln_prev = pln;
		pln->ln_next = ln;
	}
};

#endif
