/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __MATRIX_H__
#define __MATRIX_H__

#include <GL/gl.h>

#include "vector.h"

class matrix
{
	friend class vector;

 private:
	float m[16];
	static float identity[16];

 public:
	void set();
	void set(const vector *);
	void set(const vector *, const vector *);
	void set(const vector *, const vector *,
		 const vector *, const vector *);

	void invert(matrix *);
	void translate(vector *);

	void rotate_x(float);
	void rotate_y(float);
	void rotate_z(float);

	inline void glload() { glLoadMatrixf(m); }
	inline void glmult() { glMultMatrixf(m); }
};

inline void vector::get_x(const matrix *m)
{
	set(m->m[0], m->m[1], m->m[2]);
}

inline void vector::get_y(const matrix *m)
{
	set(m->m[4], m->m[5], m->m[6]);
}

inline void vector::get_z(const matrix *m)
{
	set(m->m[8], m->m[9], m->m[10]);
}

inline void vector::get_w(const matrix *m)
{
	set(m->m[12], m->m[13], m->m[14]);
}

inline void vector::transform(const matrix *m)
{
	set(m->m[0]*v[0] + m->m[4]*v[1] + m->m[8]*v[2] + m->m[12],
	    m->m[1]*v[0] + m->m[5]*v[1] + m->m[9]*v[2] + m->m[13],
	    m->m[2]*v[0] + m->m[6]*v[1] + m->m[10]*v[2] + m->m[14]);
}

inline void vector::transform(const vector *p, const matrix *m)
{
	set(m->m[0]*p->v[0] + m->m[4]*p->v[1] + m->m[8]*p->v[2] + m->m[12],
	    m->m[1]*p->v[0] + m->m[5]*p->v[1] + m->m[9]*p->v[2] + m->m[13],
	    m->m[2]*p->v[0] + m->m[6]*p->v[1] + m->m[10]*p->v[2] + m->m[14]);
}

inline void vector::rotate(const matrix *m)
{
	set(m->m[0]*v[0] + m->m[4]*v[1] + m->m[8]*v[2],
	    m->m[1]*v[0] + m->m[5]*v[1] + m->m[9]*v[2],
	    m->m[2]*v[0] + m->m[6]*v[1] + m->m[10]*v[2]);
}

inline void vector::rotate(const vector *p, const matrix *m)
{
	set(m->m[0]*p->v[0] + m->m[4]*p->v[1] + m->m[8]*p->v[2],
	    m->m[1]*p->v[0] + m->m[5]*p->v[1] + m->m[9]*p->v[2],
	    m->m[2]*p->v[0] + m->m[6]*p->v[1] + m->m[10]*p->v[2]);
}

#endif
