/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __MODEL_H__
#define __MODEL_H__

#include "matrix.h"
#include "texture.h"

class model;

class m_vertex
{
 private:
	vector m_vert, m_norm;
	vector o_vert, o_norm;

 public:

	inline void glvertex() const { m_vert.glvertex(); }
	inline void glnormal() const { m_norm.glnormal(); }

	int load(FILE *);

	inline void transform(const matrix *m)
		{ m_vert.transform(&o_vert, m);
		  m_norm.rotate(&o_norm, m); }
};

class m_vertex_index
{
private:
	const m_vertex *vtx;
	float u, v;
	
public:
	inline void glvertex() { vtx->glvertex(); }
	inline void glnormal() { vtx->glnormal(); }
	inline void gltexture() { glTexCoord2f(u, v); }

	int load(FILE *, model *);
};

class m_triangle
{
private:
	m_vertex_index vtx[3];

public:
	void draw();
	int load(FILE *, model *);
};

class m_frame
{
 private:
	vector pos, rot;

 public:
	int load(FILE *);

	inline const vector *position() const { return &pos; }
	inline const vector *rotation() const { return &rot; }
};

class m_bone
{
 private:
	unsigned nvtx;
	m_vertex *vtx;

	m_frame *fra;
	
 public:
	m_bone();
	~m_bone();

	void draw(unsigned, unsigned, float);
	int load(FILE *, unsigned);

	inline unsigned vertices() const { return nvtx; }
	inline const m_vertex *find_vertex(unsigned v) const
		{ return &vtx[v]; }
};

class model
{
 private:
	texture *tex;
	float rad;

	unsigned nfra;

	unsigned nbone;
	m_bone *bone;

	unsigned ntri;
	m_triangle *tri;

 public:
	model();
	~model();

	void draw(unsigned, unsigned, float);
	int load(const char *);

	inline unsigned frames() const { return nfra; }
	inline unsigned bones() const { return nbone; }
	inline const m_bone *find_bone(unsigned b) const
		{ return &bone[b]; }
	inline float radius() const { return rad; }
};

#endif
