/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <errno.h>

#include <GL/gl.h>

#include "config.h"
#include "misc.h"

void screenshot(unsigned width, unsigned height)
{
	FILE *fp;
	DIR *dir;
	struct dirent *ent;
	unsigned i, num = 0;
	unsigned char *pixels;
	const char *dot;
	char name[1024];

	if ( !(dot = cfg.get_string("dot_dir")) )
	{
		dot = ".";
	}

	sprintf(name, "%s/screen", dot);

	if ( (dir = opendir(name)) )
	{
		while ( (ent = readdir(dir)) )
		{
			i = strtoul(ent->d_name, 0, 10);
			if (i > num) num = i;
		}
		closedir(dir);
	}
	else if (errno == ENOENT)
	{
		mkdir(name, 0777);
	}

	pixels = (unsigned char *)alloca(width * 3);

	sprintf(name, "%s/screen/%u.ppm", dot, ++num);

	if ( (fp = fopen(name, "wb")) )
	{
		fprintf(fp, "P6\n%u %u\n255\n", width, height);

		glReadBuffer(GL_FRONT);
		for (i = height - 1; i < height; i--)
		{
			glReadPixels(0, i, width, 1, GL_RGB,
				     GL_UNSIGNED_BYTE, pixels);
			fwrite(pixels, 1, width * 3, fp);
		}

		fclose(fp);
	}
}
