/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __COMPILE_H__
#define __COMPILE_H__

#include <stdio.h>

/* configuration */

#define CPPPATH "/usr/bin/cpp"

/* types */

#define T_VOID     0
#define T_FLOAT    1
#define T_VECTOR   2
#define T_STRING   3
#define T_FUNCTION 4
#define T_ENTITY   5

/* expressions */

#define E_NOP      0
#define E_CONSTANT 1
#define E_LOCAL    2
#define E_GLOBAL   3
#define E_FUNCTION 4
#define E_DOTX     5
#define E_DOTY     6
#define E_DOTZ     7
#define E_UNARY    8
#define E_BINARY   9
#define E_ASSIGN  10
#define E_CALL    11

/* binary expressions */

#define B_ADD   1
#define B_SUB   2
#define B_MUL   3
#define B_DIV   4
#define B_EQ    5
#define B_NE    6
#define B_LT    7
#define B_GT    8
#define B_LE    9
#define B_GE   10
#define B_AND  11
#define B_OR   12
#define B_EOR  13
#define B_LAND 14
#define B_LOR  15

/* unary expressions */

#define U_NEG 1
#define U_INV 2
#define U_NOT 3

/* statment types */

#define S_BLOCK    0
#define S_EXPR     1
#define S_IF       2
#define S_UNLESS   3
#define S_WHILE    4
#define S_UNTIL    5
#define S_DO_WHILE 6
#define S_DO_UNTIL 7
#define S_FOR      8
#define S_RETURN   9

/* opcodes */

#define OP_DONE       0 /* - - - */

#define OP_LOAD_EF    1 /* e x f */
#define OP_LOAD_EV    2 /* e x v */
#define OP_LOAD_ES    3 /* e x s */
#define OP_LOAD_EN    4 /* e x n */
#define OP_LOAD_EE    5 /* e x e */
#define OP_STORE_EF   6 /* e x f */
#define OP_STORE_EV   7 /* e x v */
#define OP_STORE_ES   8 /* e x s */
#define OP_STORE_EN   9 /* e x n */
#define OP_STORE_EE  10 /* e x e */

#define OP_LOAD_NF   11 /* n x f */
#define OP_LOAD_NV   12 /* n x v */
#define OP_LOAD_NS   13 /* n x s */
#define OP_LOAD_NN   14 /* n x n */
#define OP_LOAD_NE   15 /* n x e */
#define OP_STORE_NF  16 /* n x f */
#define OP_STORE_NV  17 /* n x v */
#define OP_STORE_NS  18 /* n x s */
#define OP_STORE_NN  19 /* n x n */
#define OP_STORE_NE  20 /* n x e */

#define OP_IF_F      21 /* f a - */
#define OP_IF_V      22 /* v a - */
#define OP_IF_S      23 /* s a - */
#define OP_IF_N      24 /* n a - */
#define OP_IF_E      25 /* e a - */
#define OP_IFNOT_F   26 /* f a - */
#define OP_IFNOT_V   27 /* v a - */
#define OP_IFNOT_S   28 /* s a - */
#define OP_IFNOT_N   29 /* n a - */
#define OP_IFNOT_E   30 /* e a - */

#define OP_GOTO      31 /* - a - */
#define OP_CALL      32 /* e n - */
#define OP_RETURN    33 /* - - - */

#define OP_MOVE_F    34 /* f - f */
#define OP_MOVE_V    35 /* v - v */
#define OP_MOVE_S    36 /* s - s */
#define OP_MOVE_N    37 /* n - n */
#define OP_MOVE_E    38 /* e - e */

#define OP_MUL_F     39 /* f f f */
#define OP_MUL_V     40 /* v v f */
#define OP_MUL_FV    41 /* f v v */
#define OP_MUL_VF    42 /* v f v */
#define OP_DIV_F     43 /* f f f */
#define OP_DIV_VF    44 /* v f v */
#define OP_ADD_F     45 /* f f f */
#define OP_ADD_V     46 /* v v v */
#define OP_SUB_F     47 /* f f f */
#define OP_SUB_V     48 /* v v v */
#define OP_NEG_F     49 /* f - f */
#define OP_NEG_V     50 /* v - v */

#define OP_EQ_F      51 /* f f f */
#define OP_EQ_V      52 /* v v f */
#define OP_EQ_S      53 /* s s f */
#define OP_EQ_N      54 /* n n f */
#define OP_EQ_E      55 /* e e f */
#define OP_NE_F      56 /* f f f */
#define OP_NE_V      57 /* v v f */
#define OP_NE_S      58 /* s s f */
#define OP_NE_N      59 /* n n f */
#define OP_NE_E      60 /* e e f */
#define OP_LE_F      61 /* f f f */
#define OP_GE_F      62 /* f f f */
#define OP_LT_F      63 /* f f f */
#define OP_GT_F      64 /* f f f */

#define OP_AND_FF    65 /* f f f */
#define OP_AND_FV    66 /* f v f */
#define OP_AND_FS    67 /* f s f */
#define OP_AND_FN    68 /* f n f */
#define OP_AND_FE    69 /* f e f */
#define OP_AND_VF    70 /* v f f */
#define OP_AND_VV    71 /* v v f */
#define OP_AND_VS    72 /* v s f */
#define OP_AND_VN    73 /* v n f */
#define OP_AND_VE    74 /* v e f */
#define OP_AND_SF    75 /* s f f */
#define OP_AND_SV    76 /* s v f */
#define OP_AND_SS    77 /* s s f */
#define OP_AND_SN    78 /* s n f */
#define OP_AND_SE    79 /* s e f */
#define OP_AND_NF    80 /* n f f */
#define OP_AND_NV    81 /* n v f */
#define OP_AND_NS    82 /* n s f */
#define OP_AND_NN    83 /* n n f */
#define OP_AND_NE    84 /* n e f */
#define OP_AND_EF    85 /* e f f */
#define OP_AND_EV    86 /* e v f */
#define OP_AND_ES    87 /* e s f */
#define OP_AND_EN    88 /* e n f */
#define OP_AND_EE    89 /* e e f */
#define OP_OR_FF     90 /* f f f */
#define OP_OR_FV     91 /* f v f */
#define OP_OR_FS     92 /* f s f */
#define OP_OR_FN     94 /* f n f */
#define OP_OR_FE     93 /* f e f */
#define OP_OR_VF     95 /* v f f */
#define OP_OR_VV     96 /* v v f */
#define OP_OR_VS     97 /* v s f */
#define OP_OR_VN     98 /* v n f */
#define OP_OR_VE     99 /* v e f */
#define OP_OR_SF    100 /* s f f */
#define OP_OR_SV    101 /* s v f */
#define OP_OR_SS    102 /* s s f */
#define OP_OR_SN    103 /* s n f */
#define OP_OR_SE    104 /* s e f */
#define OP_OR_NF    105 /* n f f */
#define OP_OR_NV    106 /* n v f */
#define OP_OR_NS    107 /* n s f */
#define OP_OR_NN    108 /* n n f */
#define OP_OR_NE    109 /* n e f */
#define OP_OR_EF    110 /* e f f */
#define OP_OR_EV    111 /* e v f */
#define OP_OR_ES    112 /* e s f */
#define OP_OR_EN    113 /* e n f */
#define OP_OR_EE    114 /* e e f */
#define OP_NOT_F    115 /* f - f */
#define OP_NOT_V    116 /* v - f */
#define OP_NOT_S    117 /* s - f */
#define OP_NOT_N    118 /* n - f */
#define OP_NOT_E    119 /* e - f */

#define OP_BITAND_F 120 /* f f f */
#define OP_BITOR_F  121 /* f f f */
#define OP_BITEOR_F 122 /* f f f */
#define OP_BITNOT_F 123 /* f - f */

/* vector deninitions */

typedef struct vector
{
	float x, y, z;
} vector;

/* string definitions */

typedef struct string
{
	struct string *next;
	unsigned idx;
	const char *cstr;
} string;

string *new_string(const char *);

extern string *string_hash[256];

/* type definitions */

typedef struct type
{
	struct type *next;
	unsigned char typ;
	struct function *fn;
} type;

type *new_type(char, struct function *);

/* expression definitions */

typedef struct expression
{
	struct expression *prev;
	unsigned char func, op;
	type *typ;
	struct variable *var;
	struct function *fn;
	struct expression *left;
	struct expression *right;
} expression;

expression *new_exprssion(char, char, char);
expression *new_nop_expr(expression *);
expression *new_const_expr(struct variable *);
expression *new_variable_expr(struct expression *, string *);
expression *new_unary_expr(char, expression *);
expression *new_binary_expr(char, expression *, expression *);
expression *new_assign_expr(expression *, expression *);
expression *new_vecdot_expr(expression *, string *);
expression *new_call_expr(expression *, expression *);

/* statement definitions */

typedef struct statement
{
	struct statement *next;
	int func;
	struct expression *e1, *e2, *e3;
	struct statement *s1, *s2;
} statement;

statement *new_statement(int);

/* variable definitions */

typedef struct variable
{
	struct variable *next;
	string *name;
	type *typ;
	unsigned char cnst, used, defd;
	unsigned idx;
	union {
		float flt;
		struct vector vec;
		struct string *str;
		struct function *fn;
	} u;
} variable;

variable *new_variable(type *, string *);
variable *new_this(void);
variable *new_parameters(variable *);
void add_global_vars(type *, variable *);
void add_global_var(variable *);
void add_local_vars(type *, variable *);
void add_local_var(variable *);
variable *new_float_const(float);
variable *new_vector_const(float, float, float);
variable *new_string_const(string *);
variable *new_function_const(struct function *);
variable *find_variable(variable *, string *);

extern variable *globals;

/* instruction definitions */

typedef struct instruction
{
	unsigned char op;
	char ao, bo, co;
	union {
		struct variable *var;
		int idx;
	} a, b, c;
} instruction;

/* function definitions */

typedef struct function
{
	struct function *next;
	variable var;
	variable *params, *constants, *locals, *tmps;
	statement *body;
	instruction *instrs;
	unsigned short ninstrs, nparams, nvars;
} function;

function *new_function(type *, string *, variable *);
function *find_function(string *);
int fn_check_type(function *, function *);

extern function *functions;
extern function *this_fn;

/* misc definitions */

void compile(void);
void calc_indexes(void);
void output(FILE *);
void print_error(char *);

/* yacc definitions */

void yyerror(const char *);
void yyerrorf(const char *, ...);

int yylex(void);

#endif
