/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

D [[:digit:]]
ID [[:alpha:]_][[:alnum:]_]*
E [Ee][+-]?{D}+

%%

^[[:space:]]*"#".*$ { /* */ }

"/*" { 
	char c1, c2 = 0;

	while ((c1 = input()))
	{
		if (c1 == '/' && c2 == '*')
			break;
		c2 = c1;
	}
}

"//".*$ { /* */ }

\"(\\.|[^\\"])*\" { 
	char str[256];
	char *s = yytext;
	int n;
	
	for (n = 0; n < 256; n++)
	{
		str[n] = *(++s);
		switch (str[n])
		{
		case '\\':
			str[n] = *(++s);
			break;

		case '"':
			str[n] = '\0';
			yylval.str = new_string(str);
			return TEXT;
		}
	}

	yyerrorf("string constant too long %s", yytext);
	yylval.str = new_string("");
	return TEXT;
}

{D}+"."{D}*{E}? { yylval.flt = atof(yytext); return NUMBER; }
{D}*"."{D}+{E}? { yylval.flt = atof(yytext); return NUMBER; }
{D}+{E}? { yylval.flt = atof(yytext); return NUMBER; }

{ID} { 
	if (!strcmp(yytext, "if"))
	{
		return IF;
	}
	if (!strcmp(yytext, "unless"))
	{
		return UNLESS;
	}
	else if (!strcmp(yytext, "else"))
	{
		return ELSE;
	}
	else if (!strcmp(yytext, "for"))
	{
		return FOR;
	}
	else if (!strcmp(yytext, "do"))
	{
		return DO;
	}
	else if (!strcmp(yytext, "while"))
	{
		return WHILE;
	}
	else if (!strcmp(yytext, "until"))
	{
		return UNTIL;
	}
	else if (!strcmp(yytext, "return"))
	{
		return RETURN;
	}
	else if (!strcmp(yytext, "this"))
	{
		return THIS;
	}
	else if (!strcmp(yytext, "void"))
	{
		return VOID;
	}
	else if (!strcmp(yytext, "float"))
	{
		return FLOAT;
	}
	else if (!strcmp(yytext, "vector"))
	{
		return VECTOR;
	}
	else if (!strcmp(yytext, "string"))
	{
		return STRING;
	}
	else if (!strcmp(yytext, "entity"))
	{
		return ENTITY;
	}
	else
	{
		yylval.str = new_string(yytext);
		return ID;
	}
}

"&&" { return AND; }
"||" { return OR; }
"==" { return EQ; }
"!=" { return NE; }
"<=" { return LE; }
">=" { return GE; }

[[:space:]]+ { /* */ }

. { return yytext[0]; }

%%
