/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __MAKEMODEL_H__
#define __MAKEMODEL_H__

#include <stdio.h>

typedef struct vector
{
	float x, y, z;
}
vector;

typedef struct matrix
{
	vector x, y, z;
}
matrix;

typedef struct vertex
{
	struct vertex *next;
	struct bone *bone;
	unsigned idx;
	vector *vec;
	vector *norm;
}
vertex;

typedef struct vertex_reference
{
	vertex *vtx;
	double u, v;
}
vertex_reference;

typedef struct triangle
{
	struct triangle *next;
	vertex_reference *vtx[3];
}
triangle;

typedef struct frame
{
	struct frame *next;
	unsigned idx;
	struct vector *ang;
	struct vector *pos;
	unsigned prt;
}
frame;

typedef struct bone
{
	struct bone *next;
	struct bone *parent;
	struct bone *children;
	unsigned idx;
	vector *pos;
	vertex *verts;
	frame *frames;
}
bone;

typedef struct model
{
	unsigned frames;
	char *tex;
	bone *bones;
	triangle *tris;
	float radius;
}
model;

vertex *find_vertex(bone *, unsigned, unsigned);
vector *new_vector(float, float, float);

void calc_model(model *);
void calc_bones(bone *, bone *, unsigned);
void calc_radius(model *, bone *);
void fix_frames(bone *, unsigned);

bone *find_fixed_bone(bone *, unsigned);
frame *find_frame(bone *, unsigned);

void set_matrix(matrix *, vector *);
void rotate_matrix_x(matrix *, float);
void rotate_matrix_y(matrix *, float);
void rotate_matrix_z(matrix *, float);
void rotate_vector(vector *, vector *, matrix *);
void round_vector(vector *);

int save_model(model *, FILE *);
int save_bone(bone *, FILE *);
int save_triangle(triangle *, FILE *);
int save_vector(vector *, FILE *);

int put_str(FILE *, char *);
int put_int(FILE *, unsigned);
int put_fixed(FILE *, float);

void reindex_bone(bone *, unsigned *);

#endif
