/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

ID [[:alpha:]_][[:alnum:]_]*
D [[:digit:]]
S [+-]
E [Ee]{S}{D}+

%%

"#".*$ { /* */ }

\"(\\.|[^\\"])*\" { 
	char str[256];
	char *s = yytext;
	int n;
	
	for (n = 0; n < 256; n++)
	{
		str[n] = *(++s);
		switch (str[n])
		{
		case '\\':
			str[n] = *(++s);
			break;

		case '"':
			str[n] = '\0';
			yylval.str = strdup(str);
			return STRING;
		}
	}

	yyerror("string constant too long");
	yylval.str = "";
	return STRING;
}

{S}?{D}+"."{D}*{E}? { yylval.flt = atof(yytext); return NUMBER; }
{S}?{D}*"."{D}+{E}? { yylval.flt = atof(yytext); return NUMBER; }
{S}?{D}+{E} { yylval.flt = atof(yytext); return NUMBER; }
{S}{D}+ { yylval.flt = atof(yytext); return NUMBER; }
{D}+ { yylval.idx = atoi(yytext); return INTEGER; }

{ID} {
	if (!strcmp(yytext, "bone"))
	{
		return BONE;
	}
	else if (!strcmp(yytext, "frame"))
	{
		return FRAME;
	}
	else if (!strcmp(yytext, "position"))
	{
		return POSITION;
	}
	else if (!strcmp(yytext, "print"))
	{
		return PRINT;
	}
	else if (!strcmp(yytext, "texture"))
	{
		return TEXTURE;
	}
	else if (!strcmp(yytext, "triangles"))
	{
		return TRIANGLES;
	}
	else if (!strcmp(yytext, "vertices"))
	{
		return VERTICES;
	}
	else
	{
		yylval.str = yytext;
		return ID;
	}
}

[[:space:]]+ { /* */ }

. { return yytext[0]; }

%%
