/*
 *  Copyright (C) 2000 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "makemodel.h"

int save_model(model *mdl, FILE *fp)
{
	unsigned n;
	bone *bp;
	triangle *tp;

	if (fwrite("CJPM", 1, 4, fp) != 4)
		return 1;
	if (put_str(fp, mdl->tex))
		return 1;
	if (put_fixed(fp, mdl->radius))
		return 1;
	if (put_int(fp, mdl->frames))
		return 1;

	n = 0;
	for (bp = mdl->bones; bp; bp = bp->next)
	{
		reindex_bone(bp, &n);
	}
	if (put_int(fp, n))
		return 1;
	for (bp = mdl->bones; bp; bp = bp->next)
	{
		if (save_bone(bp, fp))
			return 1;
	}

	n = 0;
	for (tp = mdl->tris; tp; tp = tp->next)
	{
		n++;
	}
	if (put_int(fp, n))
		return 1;
	for (tp = mdl->tris; tp; tp = tp->next)
	{
		if (save_triangle(tp, fp))
			return 1;
	}

	return 0;
}

void reindex_bone(bone *bp, unsigned *ptr)
{
	bp->idx = (*ptr)++;

	for (bp = bp->children; bp; bp = bp->next)
	{
		reindex_bone(bp, ptr);
	}
}

int save_bone(bone *bp, FILE *fp)
{
	unsigned n;
	frame *frm;
	vertex *vp;

	for (frm = bp->frames; frm; frm = frm->next)
	{
		save_vector(frm->pos, fp);
		save_vector(frm->ang, fp);
	}

	n = 0; 
	for (vp = bp->verts; vp; vp = vp->next)
	{
		vp->idx = n++;
	}

	if (put_int(fp, n))
		return 1;

	for (vp = bp->verts; vp; vp = vp->next)
	{
		save_vector(vp->vec, fp);
		save_vector(vp->norm, fp);
	}

	for (bp = bp->children; bp; bp = bp->next)
	{
		if (save_bone(bp, fp))
			return 1;
	}

	return 0;
}

int save_triangle(triangle *tp, FILE *fp)
{
	unsigned n;

	for (n = 0; n < 3; n++)
	{
		vertex_reference *vr = tp->vtx[n];
		vertex *vp = vr->vtx;

		if (put_int(fp, vp->bone->idx))
			return 1;
		if (put_int(fp, vp->idx))
			return 1;
		if (put_fixed(fp, vr->u))
			return 1;
		if (put_fixed(fp, vr->v))
			return 1;
	}

	return 0;
}

int save_vector(vector *vp, FILE *fp)
{
	if (put_fixed(fp, vp->x))
		return 1;
	if (put_fixed(fp, vp->y))
		return 1;
	if (put_fixed(fp, vp->z))
		return 1;
	return 0;
}

int put_str(FILE *fp, char *str)
{
	unsigned n = strlen(str);

	if (n > 255) n = 255;

	if (fputc(n, fp) == EOF)
		return 1;
	if (fwrite(str, 1, n, fp) != n)
		return 1;
	return 0;
}

int put_int(FILE *fp, unsigned i)
{
	unsigned char c[2];

	c[0] = i;
	c[1] = (i >> 8);

	if (fwrite(c, 1, 2, fp) != 2)
		return 1;
	return 0;
}

int put_fixed(FILE *fp, float f)
{
	unsigned char c[2];
	float n = floor(f += (1.0 / 512.0));

	c[0] = (unsigned)(n);
	c[1] = (unsigned)floor((f - n) * 256.0);

	if (fwrite(&c, 1, 2, fp) != 2)
		return 1;
	return 0;
}
