#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

class triangle;
class landscape;

class vector
{
 public:
	int x, y, z;

	inline void set(vector *v)
		{ x = v->x; y = v->y; z = v->z; }
	inline void set(int i, int j, int k)
		{ x = i; y = j; z = k; }
	inline void sub(vector *l,vector *r)
		{ x = l->x - r->x; y = l->y - r->y; z = l->z - r->z; }
	inline void cross(vector *l,vector *r)
		{ set(l->y * r->z - l->z * r->y,
                      l->z * r->x - l->x * r->z,
                      l->x * r->y - l->y * r->x); }

	int calc(float, float, float);

};

class vertex : public vector
{
 public:
	int u, v;

	int save(FILE *, int);
};

class triangle
{
 public:
	triangle *next;

 private:
	vertex vtx[3];

 public:
	triangle(vertex *, vertex *, vertex *, int);
	~triangle();

	int save(FILE *, int);

	void min_max(int *, int *);
};

class square 
{
 private:
	triangle *tris;
	int z;

 public:
	square();
	~square();

	void add_triangle(vertex *, vertex *, vertex *, int);

	int save(FILE *);

	void fix_heights();
};

class landscape
{
 private:
	const char *tex;
	square sqr[256][256];
	unsigned n_tri;
	unsigned n_out;
	unsigned n_bad;
	unsigned n_sml;

 public:
	landscape();
	~landscape();

	void set_texture(const char *);

	int load_triangles(const char *, int);
	int save(const char *);

	void fix_heights();
	void summary();
};

int main(int argc, char **argv)
{
	landscape map;
	unsigned tile = 0;

	if (argc < 4 || argc > 5)
	{
		fprintf(stderr,
			"Usgae: %s <output> <triangles> <texture> [<exp>]\n",
			argv[0]);
		return 1;
	}

	map.set_texture(argv[3]);

	if (argc > 4) tile = atoi(argv[4]);
	if (tile > 2) tile = 0;
	tile = 1U << tile;

	if (map.load_triangles(argv[2], tile))
	{
		fprintf(stderr, "Failed to load triangles \"%s \"\n", argv[2]);
		return 1;
	}

	map.fix_heights();

	if (map.save(argv[1]))
	{
		fprintf(stderr, "Failed to save map \"%s\"\n", argv[1]);
		return 1;
	}

	map.summary();

	return 0;
}


landscape::landscape()
{
	tex = "";
	n_tri = 0;
	n_out = 0;
	n_bad = 0;
	n_sml = 0;
}

landscape::~landscape()
{
}

void landscape::set_texture(const char *name)
{
	const char *s;

	s = strrchr(name, '/');
	s = s ? s+1 : name;

	tex = s;
}

int landscape::load_triangles(const char *file, int tile)
{
	FILE *fp;
	char buffer[1024];
	float x0, y0, z0;
	float x1, y1, z1;
	float x2, y2, z2;
	int x, y, z, n;
	vertex v0, v1, v2;
	vector va, vb, vn;
	
	if ( !(fp = fopen(file, "r")) )
		return -1;

	while (fgets(buffer, 1024, fp))
	{
		if (sscanf(buffer, "%f %f %f %f %f %f %f %f %f",
			   &x0, &y0, &z0, &x1, &y1, &z1, 
			   &x2, &y2, &z2) != 9)
		{
			fclose(fp);
			return -1;
		}

		n_tri++;

		if (v0.calc(x0, y0, z0) ||
		    v1.calc(x1, y1, z1) ||
		    v2.calc(x2, y2, z2))
		{
			n_out++;
			continue;
		}

		va.sub(&v1, &v0);
		vb.sub(&v2, &v0);
		vn.cross(&va, &vb);

		if (vn.x == 0 && vn.y == 0 && vn.z == 0)
		{
			n_sml++;
			continue;
		}

		x = abs(vn.x);
		y = abs(vn.y);
		z = abs(vn.z);

		v0.u = v0.x; v0.v = v0.y;
		v1.u = v1.x; v1.v = v1.y;
		v2.u = v2.x; v2.v = v2.y;
		n = vn.z;

		if (x > y)
		{
			if (x > z)
			{
				v0.u = -v0.y; v0.v = -v0.z;
				v1.u = -v1.y; v1.v = -v1.z;
				v2.u = -v2.y; v2.v = -v2.z;
				n = vn.x;
			}
		}
		else
		{
			if (y > z)
			{
				v0.u = v0.x; v0.v = -v0.z;
				v1.u = v1.x; v1.v = -v1.z;
				v2.u = v2.x; v2.v = -v2.z;
				n = vn.y;
			}
		}

		if (n >= 0)
		{
			v0.u = -v0.u;
			v1.u = -v1.u;
			v2.u = -v2.u;
		}

		x = ((v0.x + v1.x + v2.x) / 3) & -16;
		y = ((v0.y + v1.y + v2.y) / 3) & -16;

		if (vn.x >= 0 &&
		    (v0.x & 15) == 0 &&
		    v1.x == v0.x &&
		    v2.x == v0.x)
		{
			x -= 16;
		}

		if (vn.y >= 0 &&
		    (v0.y & 15) == 0 &&
		    v1.y == v0.y &&
		    v2.y == v0.y)
		{
			y -= 16;
		}

		if (x < 0) x = 0;
		if (y < 0) x = 0;

		if (x > 4080) x = 4080;
		if (y > 4080) y = 4080;

		v0.x -= x; v0.y -= y;
		v1.x -= x; v1.y -= y;
		v2.x -= x; v2.y -= y;

		if (v0.x < 0 || v0.x > 16 || v0.y < 0 || v0.y > 16 ||
		    v1.x < 0 || v1.x > 16 || v1.y < 0 || v1.y > 16 || 
		    v2.x < 0 || v2.x > 16 || v2.y < 0 || v2.y > 16)
		{
			n_bad++;
			continue;
		}

		sqr[y>>4][x>>4].add_triangle(&v0, &v1, &v2, tile);
	}

	if (fclose(fp))
		return -1;

	return 0;
}

int landscape::save(const char *file)
{
	FILE *fp;
	unsigned i, j;

	if ( !(fp = fopen(file, "w")) )
		return -1;

	if (fwrite("CJPL", 1, 4, fp) != 4)
	{
		fclose(fp);
		return -1;
	}

	if (fputc(1, fp) != 1)
	{
		fclose(fp);
		return -1;
	}

	i = strlen(tex);

	if (fputc(i, fp) != int(i))
		return -1;

	if (fwrite(tex, 1, i, fp) != i)
		return -1;

	for (j = 0; j < 256; j++)
	{
		for (i = 0; i < 256; i++)
		{
			if (sqr[j][i].save(fp))
			{
				fclose(fp);
				return -1;
			}
		}
	}

	if (fclose(fp))
		return -1;

	return 0;
}

void landscape::fix_heights()
{
	unsigned i, j;

	for (j = 0; j < 256; j++)
	{
		for (i = 0; i < 256; i++)
		{
			sqr[j][i].fix_heights();
		}
	}
}

void landscape::summary()
{
	printf("%u triangles\n", n_tri);
	printf("%u dropped (%u outside map)\n", n_out + n_bad + n_sml, n_out);
	printf("           (%u intersect grid)\n", n_bad);
	printf("           (%u too small)\n", n_sml);
}

square::square()
{
	tris = 0;
	z = 0;
}

square::~square()
{
	while (tris)
	{
		triangle *t = tris;
		tris = t->next;
		delete t;
	}
}

void square::add_triangle(vertex *v0, vertex *v1, vertex *v2, int tile)
{
	triangle *t = new triangle(v0, v1, v2, tile);

	t->next = tris;
	tris = t;
}

int square::save(FILE *fp)
{
	unsigned i;
	triangle *t;

	if (fputc(z, fp) != z)
                return -1;

	i = 0;
	for (t = tris; t; t = t->next)
	{
		++i;
	}

	if (fputc(i, fp) != int(i))
                return -1;

	for (t = tris; t; t = t->next)
	{
		if (t->save(fp, z))
			return -1;
	}

	return 0;
}

void square::fix_heights()
{
	triangle *t;
	int min = z;
	int max = z;

	for (t = tris; t; t = t->next)
	{
		t->min_max(&min, &max);
	}

	if ((max - z) > 127) z = max - 127;
	if ((min - z) < -128) z = min + 128;
}

triangle::triangle(vertex *v0, vertex *v1, vertex *v2, int tile)
{
	unsigned i;
	int mu, mv;

	next = 0;

	vtx[0].set(v0);
	vtx[1].set(v1);
	vtx[2].set(v2);

	vtx[0].u = (v0->u * 4 / tile);
	vtx[0].v = (v0->v * 4 / tile);

	vtx[1].u = (v1->u * 4 / tile);
	vtx[1].v = (v1->v * 4 / tile);

	vtx[2].u = (v2->u * 4 / tile);
	vtx[2].v = (v2->v * 4 / tile);

	mu = vtx[0].u;
	mv = vtx[0].v;
	
	for (i = 1; i < 3; i++)
	{
		if (vtx[i].u < mu) mu = vtx[i].u;
		if (vtx[i].v < mv) mv = vtx[i].v;
	}

	mu &= -64;
	mv &= -64;

	for (i = 0; i < 3; i++)
	{
		vtx[i].u -= mu;
		vtx[i].v -= mv;
	}
}

triangle::~triangle()
{
}

int triangle::save(FILE *fp, int z)
{
	unsigned i;

	if (fputc(1, fp) != 1)
		return -1;

	for (i = 0; i < 3; i++)
	{
		if (vtx[i].save(fp, z))
			return -1;
	}

	return 0;
}

void triangle::min_max(int *minp, int *maxp)
{
	int min = *minp;
	int max = *maxp;
	unsigned i;

	for (i = 0; i < 3; i++)
	{
		int z = vtx[i].z;

		if (z < min) min = z;
		if (z > max) max = z;
	}

	*minp = min;
	*maxp = max;
}

int vertex::save(FILE *fp, int h)
{
	struct {
		signed char x, y, z;
		unsigned char u, v;
	} data;

	data.x = x;
	data.y = y;
	data.z = z - h;
	data.u = u;
	data.v = v;

	if (fwrite(&data, 1, 5, fp) != 5)
		return -1;

	return 0;
}

int vector::calc(float i, float j, float k)
{
	x = int(floor(((i + 128.0) * 16.0) + 0.5));
	y = int(floor(((128.0 - k) * 16.0) + 0.5));
	z = int(floor((j * 16.0) + 0.5));

	if (x < 0 || x > 4096 ||
	    y < 0 || y > 4096 ||
	    z < -128 || z > 382)
		return -1;

	return 0;
}
